@@ -29,6 +29,8 @@ public:
 class apt_bool_widget:public apt_config_widget, public vs_checkbutton
 {
   std::string item; // the config-item we're associated with.
+
+  bool my_default;
 public:
   apt_bool_widget(std::string _label, std::string _item, bool _default);
 
@@ -38,6 +40,8 @@ public:
 class apt_string_widget:public apt_config_widget, public vs_editline
 {
   std::string item; // the config-item we're associated with.
+
+  std::string my_default;
 public:
   apt_string_widget(std::string _item, std::string _default);
 
