@@ -132,8 +132,6 @@ option_item dependency_options[]={
 	      PACKAGE "::Auto-Fix-Broken", true),
   option_item(N_("Install Recommended packages automatically"),
 	      PACKAGE "::Recommends-Important", true),
-  option_item(N_("Install Suggested packages automatically"),
-	      PACKAGE "::Suggests-Important", false),
   option_item(N_("Remove unused packages automatically"),
 	      PACKAGE "::Delete-Unused", true),
   option_item(N_("Automatically remove unused packages matching this filter"),
