@@ -341,6 +341,71 @@ static bool prompt_essential()
   return true;
 }
 
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+/** Checks for trust violations and displays a big fat warning if any exist.
+ *
+ *  \return true if everything is OK or the user overrode the warning.
+ */
+static bool prompt_trust()
+{
+  pkgvector untrusted;
+
+  for(pkgCache::PkgIterator pkg=(*apt_cache_file)->PkgBegin();
+      !pkg.end(); ++pkg)
+    {
+      pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
+
+      if(state.Install())
+	{
+	  pkgCache::VerIterator curr=pkg.CurrentVer();
+	  pkgCache::VerIterator cand=state.InstVerIter(*apt_cache_file);
+
+	  if((curr.end() || package_trusted(curr)) &&
+	     !package_trusted(cand))
+	    untrusted.push_back(pkg);
+	}
+    }
+
+  if(!untrusted.empty())
+    {
+      printf(_("WARNING: untrusted versions of the following packages will be installed!\n\n"
+	       "Untrusted packages could compromise your system's security.\n"
+	       "You should only proceed with the installation if you are certain that\n"
+	       "this is what you want to do.\n\n"));
+
+      cmdline_show_pkglist(untrusted);
+
+      const string okstr=_("Yes"), abortstr=_("No");
+
+      while(1)
+	{
+	  printf(_("Should I go ahead and install the packages anyway?\n"));
+	  printf(_("To continue, enter \"%s\"; to abort, enter \"%s\": "), okstr.c_str(), abortstr.c_str());
+	  char buf[1024];
+	  cin.getline(buf, 1023);
+	  buf[1023]='\0';
+
+	  const bool is_ok=(strncasecmp(okstr.c_str(), buf, okstr.size())==0);
+	  const bool is_abort=(strncasecmp(abortstr.c_str(), buf, abortstr.size())==0);
+
+	  const bool rval=is_ok;
+
+	  if(!is_ok && !is_abort)
+	    printf(_("Unrecognized input.  Enter either \"%s\" or \"%s\".\n"), okstr.c_str(), abortstr.c_str());
+	  else
+	    return rval;
+	}
+    }
+
+  return true;
+}
+#else
+static bool prompt_trust()
+{
+  return true;
+}
+#endif
+
 /** Displays a preview of the stuff to be done -- like apt-get, it collects
  *  all the "stuff to install" in one place.
  *
@@ -756,7 +821,7 @@ bool cmdline_do_prompt(bool as_upgrade,
 	}
 
       // Note: only show the prompt if we're planning to continue.
-      if(rval && !prompt_essential())
+      if(rval && (!prompt_essential() || !prompt_trust()))
 	{
 	  rval=false;
 	  cont=true;
