@@ -40,7 +40,7 @@ void setup_package_deps(const pkgCache::PkgIterator &pkg,
 			pkg_signal *sig,
 			bool reverse=false);
 
-extern template void setup_package_deps<pkg_subtree>(const pkgCache::PkgIterator &pkg, const pkgCache::VerIterator &ver, pkg_subtree *tree, pkg_signal *sig, bool reverse=false);
+extern template void setup_package_deps<pkg_subtree>(const pkgCache::PkgIterator &pkg, const pkgCache::VerIterator &ver, pkg_subtree *tree, pkg_signal *sig, bool reverse);
 
 // Each package added to the following policy will expand to a tree listing its
 // dependencies.
