@@ -8,10 +8,11 @@
 
 #include "aptitude.h"
 #include "ui.h"
-#include "cmdline.h"
 
-#include "generic/apt.h"
-#include "generic/config_signal.h"
+#include <cmdline/cmdline_progress.h>
+
+#include <generic/apt.h>
+#include <generic/config_signal.h>
 
 #include "download_manager.h"
 
@@ -108,6 +109,7 @@ bool do_pkglist_update(OpProgress *load_progress,
   download_manager *download_progress;
   if(!text_download)
     download_progress=gen_download_progress(false, _("Updating package lists"),
+					    _("View the progress of the package list update"),
 					    NULL);
   else
     download_progress=gen_cmdline_download_progress();
@@ -209,6 +211,98 @@ public:
   }
 };
 
+static bool do_log(const string &log, const loglist &changed_packages)
+{
+  FILE *f=NULL;
+
+  if(log[0]=='|')
+    f=popen(log.c_str()+1, "w");
+  else
+    f=fopen(log.c_str(), "a");
+
+  if(!f)
+    {
+      _error->Errno(_("Unable to open %s to log actions"), log.c_str());
+
+      return false;
+    }
+
+  time_t curtime=time(NULL);
+  fprintf(f, "Aptitude " VERSION ": %s\n%s\n\n", _("log report"), ctime(&curtime));
+  fprintf(f, _("IMPORTANT: this log only lists intended actions; actions which fail due to\ndpkg problems may not be completed.\n\n"));
+  fprintf(f, _("Will install %li packages, and remove %li packages.\n"), (*apt_cache_file)->InstCount(), (*apt_cache_file)->DelCount());
+  if((*apt_cache_file)->UsrSize()>0)
+    fprintf(f, _("%sB of disk space will be used\n"),
+	    SizeToStr((*apt_cache_file)->UsrSize()).c_str());
+  else if((*apt_cache_file)->UsrSize()<0)
+    fprintf(f, _("%sB bytes of disk space will be freed\n"),
+	    SizeToStr((*apt_cache_file)->UsrSize()).c_str());
+  fprintf(f, "===============================================================================\n");
+  // FIXME: sort this by something useful
+  for(loglist::const_iterator i=changed_packages.begin();
+      i!=changed_packages.end(); ++i)
+    {
+      if(i->second==pkg_upgrade)
+	fprintf(f, _("[UPGRADE] %s %s -> %s\n"), i->first.Name(),
+		i->first.CurrentVer().VerStr(),
+		(*apt_cache_file)[i->first].CandidateVerIter(*apt_cache_file).VerStr());
+      else if(i->second==pkg_downgrade)
+	fprintf(f, _("[DOWNGRADE] %s %s -> %s\n"), i->first.Name(),
+		i->first.CurrentVer().VerStr(),
+		(*apt_cache_file)[i->first].CandidateVerIter(*apt_cache_file).VerStr());
+      else
+	if(i->second!=pkg_unchanged)
+	  {
+	    const char *tag=NULL;
+	    switch(i->second)
+	      {
+	      case pkg_remove:
+		tag=_("REMOVE");
+		break;
+		//case pkg_upgrade:
+		//tag=_("UPGRADE");
+		//break;
+	      case pkg_install:
+		tag=_("INSTALL");
+		break;
+	      case pkg_reinstall:
+		tag=_("REINSTALL");
+		break;
+	      case pkg_hold:
+		tag=_("HOLD");
+		break;
+	      case pkg_broken:
+		tag=_("BROKEN");
+		break;
+	      case pkg_unused_remove:
+		tag=_("REMOVE, NOT USED");
+		break;
+	      case pkg_auto_remove:
+		tag=_("REMOVE, DEPENDENCIES");
+		break;
+	      case pkg_auto_install:
+		tag=_("INSTALL, DEPENDENCIES");
+		break;
+	      case pkg_auto_hold:
+		tag=_("HOLD, DEPENDENCIES");
+	      default:
+		tag=_("????????");
+		break;
+	      }
+
+	    fprintf(f, _("[%s] %s\n"), tag, i->first.Name());
+	  }
+    }
+  fprintf(f, _("===============================================================================\n\nLog complete.\n"));
+
+  if(log[0]=='|')
+    pclose(f);
+  else
+    fclose(f);
+
+  return true;
+}
+
 bool do_install_run(OpProgress *load_progress,
 		    bool text_download, bool download_only)
 {
@@ -235,6 +329,7 @@ bool do_install_run(OpProgress *load_progress,
   if(!text_download)
     download_progress=gen_download_progress(false,
 					    _("Downloading packages"),
+					    _("View the progress of the package download"),
 					    SigC::slot(abortable,
 						       &downloader::abort));
   else
@@ -322,29 +417,25 @@ bool do_install_run(OpProgress *load_progress,
 	  break;
 	}
 
-      string log=aptcfg->Find(PACKAGE "::Log", "/var/log/" PACKAGE);
-      if(log!="")
+      vector<string> logs;
+
+      string main_log=aptcfg->Find(PACKAGE "::Log", "/var/log/" PACKAGE);
+
+      if(!main_log.empty())
+	logs.push_back(main_log);
+
+      const Configuration::Item *parent=aptcfg->Tree(PACKAGE "::Log");
+
+      if(parent!=NULL)
+	for(const Configuration::Item *curr=parent->Child;
+	    curr!=NULL; curr=curr->Next)
+	  {
+	    if(!curr->Value.empty())
+	      logs.push_back(curr->Value);
+	  }
+
+      if(!logs.empty())
 	{
-	  FILE *f=NULL;
-	  // Hmmm.  popen is nice, but I don't like libc's file-handling routines..
-	  // system calls all the way!! :P
-	  // (but for now..I don't want 50 lines of code to deal with fork() and
-	  //  associated stuff..)
-
-	  if(log[0]=='|')
-	    f=popen(log.c_str()+1, "w");
-	  else
-	    f=fopen(log.c_str(), "a");
-
-	  if(!f)
-	    {
-	      _error->Errno(_("Unable to open %s to log actions"), log.c_str());
-
-	      ok=false;
-
-	      break;
-	    }
-
 	  loglist changed_packages;
 	  for(pkgCache::PkgIterator i=(*apt_cache_file)->PkgBegin(); !i.end(); i++)
 	    {
@@ -355,78 +446,8 @@ bool do_install_run(OpProgress *load_progress,
 
 	  changed_packages.sort(log_sorter());
 
-	  time_t curtime=time(NULL);
-	  fprintf(f, "Aptitude " VERSION ": %s\n%s\n\n", _("log report"), ctime(&curtime));
-	  fprintf(f, _("IMPORTANT: this log only lists intended actions; actions which fail due to\ndpkg problems may not be completed.\n\n"));
-	  fprintf(f, _("Will install %li packages, and remove %li packages.\n"), (*apt_cache_file)->InstCount(), (*apt_cache_file)->DelCount());
-	  if((*apt_cache_file)->UsrSize()>0)
-	    fprintf(f, _("%sB of disk space will be used\n"),
-		    SizeToStr((*apt_cache_file)->UsrSize()).c_str());
-	  else if((*apt_cache_file)->UsrSize()<0)
-	    fprintf(f, _("%sB bytes of disk space will be freed\n"),
-		    SizeToStr((*apt_cache_file)->UsrSize()).c_str());
-	  fprintf(f, "===============================================================================\n");
-	  // FIXME: sort this by something useful
-	  for(loglist::iterator i=changed_packages.begin();
-	      i!=changed_packages.end(); ++i)
-	    {
-	      if(i->second==pkg_upgrade)
-		fprintf(f, _("[UPGRADE] %s %s -> %s\n"), i->first.Name(),
-			i->first.CurrentVer().VerStr(),
-			(*apt_cache_file)[i->first].CandidateVerIter(*apt_cache_file).VerStr());
-	      else if(i->second==pkg_downgrade)
-		fprintf(f, _("[DOWNGRADE] %s %s -> %s\n"), i->first.Name(),
-			i->first.CurrentVer().VerStr(),
-			(*apt_cache_file)[i->first].CandidateVerIter(*apt_cache_file).VerStr());
-	      else
-		if(i->second!=pkg_unchanged)
-		  {
-		    const char *tag=NULL;
-		    switch(i->second)
-		      {
-		      case pkg_remove:
-			tag=_("REMOVE");
-			break;
-			//case pkg_upgrade:
-			//tag=_("UPGRADE");
-			//break;
-		      case pkg_install:
-			tag=_("INSTALL");
-			break;
-		      case pkg_reinstall:
-			tag=_("REINSTALL");
-			break;
-		      case pkg_hold:
-			tag=_("HOLD");
-			break;
-		      case pkg_broken:
-			tag=_("BROKEN");
-			break;
-		      case pkg_unused_remove:
-			tag=_("REMOVE, NOT USED");
-			break;
-		      case pkg_auto_remove:
-			tag=_("REMOVE, DEPENDENCIES");
-			break;
-		      case pkg_auto_install:
-			tag=_("INSTALL, DEPENDENCIES");
-			break;
-		      case pkg_auto_hold:
-			tag=_("HOLD, DEPENDENCIES");
-		      default:
-			tag=_("????????");
-			break;
-		      }
-
-		    fprintf(f, _("[%s] %s\n"), tag, i->first.Name());
-		  }
-	    }
-	  fprintf(f, _("===============================================================================\n\nLog complete.\n"));
-
-	  if(log[0]=='|')
-	    pclose(f);
-	  else
-	    fclose(f);
+	  for(vector<string>::iterator i=logs.begin(); i!=logs.end(); ++i)
+	    do_log(*i, changed_packages);
 	}
 
       if(!text_download)
