@@ -1,6 +1,6 @@
 // download_item.cc
 //
-//  Copyright 1999 Daniel Burrows
+//  Copyright 1999, 2004, 2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "download_item.h"
 
 #include <apt-pkg/acquire-worker.h>
+#include <apt-pkg/init.h> // For the APT version number.
 #include <apt-pkg/strutl.h>
 
 #include "vscreen/vs_tree.h"
@@ -104,6 +105,11 @@ void download_item::paint(vs_tree *win, int y, bool hierarchical)
     case pkgAcquire::Item::StatError:
       output+=item.Owner->ErrorText;
       break;
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+    case pkgAcquire::Item::StatAuthError:
+      output+=item.Owner->ErrorText;
+      break;
+#endif
     }
 
   win->mvaddnstr(y, 0, output.c_str(), width);
