@@ -230,7 +230,7 @@ void download_screen::paint_status()
 
 bool download_screen::handle_char(chtype ch)
 {
-  if(global_bindings.key_matches(ch, "ExitScreen") || global_bindings.key_matches(ch, "AbortScreen"))
+  if(global_bindings.key_matches(ch, "Quit"))
     cancelled=true;
   else
     return vs_tree::handle_char(ch);
