@@ -7,6 +7,8 @@
 #include <apt-pkg/error.h>
 
 #include <sigc++/bind.h>
+#include <sigc++/object.h>
+#include <sigc++/object_slot.h>
 
 #include <generic/apt.h>
 
@@ -14,6 +16,7 @@
 #include <vscreen/vs_subtree.h>
 
 using namespace std;
+using namespace __gnu_cxx;
 
 // Stores a group (name) and a Y/N state
 class vs_hier_editor::vs_hier_item:public SigC::Object, public vs_treeitem
@@ -127,7 +130,7 @@ public:
 
 vs_hier_editor::vs_hier_editor():item(NULL)
 {
-  hier_reloaded.connect(slot(this, &vs_hier_editor::handle_reload));
+  hier_reloaded.connect(slot(*this, &vs_hier_editor::handle_reload));
 }
 
 void vs_hier_editor::handle_reload()
@@ -192,7 +195,7 @@ void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
 		  ++i)
 		{
 		  vs_hier_item *tmp=new vs_hier_item(&i->second, item);
-		  commit_changes.connect(SigC::slot(tmp, &vs_hier_item::commit));
+		  commit_changes.connect(SigC::slot(*tmp, &vs_hier_item::commit));
 
 		  newroot->add_child(tmp);
 		  items.push_back(tmp);
@@ -214,7 +217,7 @@ void vs_hier_editor::set_package(const pkgCache::PkgIterator &pkg)
     {
       string name=pkg.end()?"":pkg.Name();
 
-      shown_conn=shown_sig.connect(bind(slot(this,
+      shown_conn=shown_sig.connect(bind(slot(*this,
 					     (void (vs_hier_editor::*) (string)) &vs_hier_editor::set_package),
 					name));
     }
