@@ -137,7 +137,7 @@ public:
   void undo()
   {
     for(vector<pkgCache::PkgIterator>::iterator i=packages.begin(); i!=packages.end(); i++)
-      owner->get_ext_state(*i).new_package=true;
+      owner->set_new_flag(*i, true);
 
     // Hack to make all the trees rebuild themselves.
     cache_reloaded();
@@ -170,15 +170,16 @@ aptitudeDepCache::aptitudeDepCache(pkgCache *Cache, Policy *Plcy)
   // When the "install recommended packages" flag changes, collect garbage.
   //
   // (oh god make the indentation pain stop)
-  aptcfg->connectB(PACKAGE "::Recommends-Important",
-		   SigC::hide<bool>(SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
-											  &aptitudeDepCache::mark_and_sweep),
-									       NULL)));
+  aptcfg->connect(PACKAGE "::Recommends-Important",
+		  SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
+									&aptitudeDepCache::mark_and_sweep),
+							     NULL));
 
-  aptcfg->connectB(PACKAGE "::Suggests-Important",
-		   SigC::hide<bool>(SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
-											  &aptitudeDepCache::mark_and_sweep),
-									       NULL)));
+  // sim.
+  aptcfg->connect(PACKAGE "::Keep-Important",
+		  SigC::bind<undo_group*, void, undo_group*>(SigC::slot(*this,
+									&aptitudeDepCache::mark_and_sweep),
+							     NULL));
 }
 
 bool aptitudeDepCache::Init(OpProgress *Prog, bool WithLock, bool do_initselections, const char *status_fname)
@@ -271,6 +272,7 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      aptitude_state &pkg_state=get_ext_state(pkg);
 
 	      section.FindFlag("Unseen", tmp, 1);
+
 	      pkg_state.new_package=(tmp==1);
 
 	      tmp=0;
@@ -312,6 +314,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 
   Prog.OverallProgress(0, Head().PackageCount, 1, _("Initializing package states"));
 
+  new_package_count=0;
+
   // Act on them
   for(pkgCache::PkgIterator i=PkgBegin(); !i.end(); i++)
     {
@@ -320,6 +324,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 
       if(initial_open) // Don't make everything "new".
 	estate.new_package=false;
+      else if(!i.VersionList().end() && estate.new_package)
+	++new_package_count;
 
       switch(estate.selection_state)
 	{
@@ -388,8 +394,15 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
   duplicate_cache(&backup_state);
 
   if(aptcfg->FindB(PACKAGE "::Auto-Upgrade", false) && do_initselections)
-    mark_all_upgradable(aptcfg->FindB("Auto-Install", true),
+    mark_all_upgradable(aptcfg->FindB(PACKAGE "::Auto-Install", true),
 			NULL);
+  else
+    {
+      // Normally this was done in the mark_all_upgradable, but we no
+      // longer do that by default.
+      mark_and_sweep(NULL);
+      cleanup_after_change(NULL);
+    }
 
   duplicate_cache(&backup_state);
 
@@ -401,6 +414,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
 					   undo_group *undo)
 {
+  begin_action_group();
+
   for(int iter=0; iter==0 || (iter==1 && with_autoinst); ++iter)
     {
       // Do this twice, only turning auto-install on the second time.
@@ -456,14 +471,8 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
 	}
     }
 
-  if(group_level==0)
-    {
-      mark_and_sweep(undo);
-
-      cleanup_after_change(undo);
-
-      duplicate_cache(&backup_state);
-    }
+  // will handle setting undos, mark-and-sweep, etc:
+  end_action_group(undo);
 }
 
 // If fd is -1, just write unconditionally to the given fd.
@@ -586,6 +595,23 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
   return true;
 }
 
+void aptitudeDepCache::set_new_flag(const pkgCache::PkgIterator &pkg,
+				    bool is_new)
+{
+  aptitude_state &estate=get_ext_state(pkg);
+
+  if(estate.new_package && !is_new)
+    {
+      --new_package_count;
+      estate.new_package=is_new;
+    }
+  else if(!estate.new_package && is_new)
+    {
+      ++new_package_count;
+      estate.new_package=is_new;
+    }
+}
+
 void aptitudeDepCache::forget_new(undoable **undoer)
 {
   forget_undoer *undo=undoer?new forget_undoer(this):NULL;
@@ -599,6 +625,8 @@ void aptitudeDepCache::forget_new(undoable **undoer)
 	  undo->add_item(i);
       }
 
+  new_package_count=0;
+
   if(undoer && undo && !undo->empty())
     *undoer=undo;
   else
@@ -629,7 +657,8 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 	 package_states[pkg->ID].selection_state!=backup_state.AptitudeState[pkg->ID].selection_state ||
 	 package_states[pkg->ID].reinstall!=backup_state.AptitudeState[pkg->ID].reinstall ||
 	 package_states[pkg->ID].install_reason!=backup_state.AptitudeState[pkg->ID].install_reason ||
-	 package_states[pkg->ID].remove_reason!=backup_state.AptitudeState[pkg->ID].remove_reason)
+	 package_states[pkg->ID].remove_reason!=backup_state.AptitudeState[pkg->ID].remove_reason ||
+	 package_states[pkg->ID].forbidver!=backup_state.AptitudeState[pkg->ID].forbidver)
 	{
 	  int n=pkg->ID;
 	  n=n;
@@ -662,7 +691,12 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 		     package_states[pkg->ID].selection_state==pkgCache::State::Purge)
 		    package_states[pkg->ID].install_reason=libapt;
 
-		  package_states[pkg->ID].selection_state=pkgCache::State::Install;
+		  if(!pkg.CurrentVer().end())
+		    package_states[pkg->ID].selection_state=pkgCache::State::Install;
+		  else if(pkg->CurrentState==pkgCache::State::NotInstalled)
+		    package_states[pkg->ID].selection_state=pkgCache::State::Purge;
+		  else
+		    package_states[pkg->ID].selection_state=pkgCache::State::DeInstall;
 		  break;
 		case ModeInstall:
 		  if(package_states[pkg->ID].selection_state!=pkgCache::State::Install)
@@ -768,9 +802,11 @@ void aptitudeDepCache::internal_mark_keep(const PkgIterator &Pkg, bool Soft, boo
   pkgDepCache::SetReInstall(Pkg, false);
   get_ext_state(Pkg).reinstall=false;
 
-  // On a garbage package, this has the effect of making it manually
-  // installed.
-  if(!Pkg.CurrentVer().end() && get_ext_state(Pkg).garbage)
+  // On an unused package, this has the effect of making it manually
+  // installed.  (check the current version to avoid adjusting purged
+  // packages)
+  if((*this)[Pkg].Delete() && !Pkg.CurrentVer().end() &&
+     get_ext_state(Pkg).remove_reason==unused)
     get_ext_state(Pkg).install_reason=manual;
 
   if(Pkg.CurrentVer().end())
@@ -884,7 +920,7 @@ void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
     }
 }
 
-bool aptitudeDepCache::all_upgrade(undo_group *undo)
+bool aptitudeDepCache::all_upgrade(bool with_autoinst, undo_group *undo)
 {
   pkgProblemResolver fixer(this);
 
@@ -898,7 +934,7 @@ bool aptitudeDepCache::all_upgrade(undo_group *undo)
 
       if(!is_held(pkg) &&
 	 !pkg.CurrentVer().end() && !(*this)[pkg].Install())
-	MarkInstall(pkg, false);
+	MarkInstall(pkg, with_autoinst);
     }
 
   bool rval=fixer.ResolveByKeep();
@@ -1031,9 +1067,7 @@ void aptitudeDepCache::end_action_group(undo_group *undo)
 {
   assert(group_level>0);
 
-  group_level--;
-
-  if(group_level==0)
+  if(group_level==1)
     {
       mark_and_sweep(undo);
 
@@ -1041,6 +1075,8 @@ void aptitudeDepCache::end_action_group(undo_group *undo)
 
       duplicate_cache(&backup_state);
     }
+
+  group_level--;
 }
 
 const aptitudeDepCache::apt_state_snapshot *aptitudeDepCache::snapshot_apt_state()
@@ -1075,25 +1111,31 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
 // If a package version is not installed and is not going to be installed, it
 // is not visited further.  Otherwise, the appropriate mark is set.
 //
-//  FIXME: follow Recommends and Suggests iff the appropriate stuff is set.
+//  FIXME: follow Recommends iff the appropriate stuff is set.
 void aptitudeDepCache::mark_package(const PkgIterator &pkg,
-				    const VerIterator &ver)
+				    const VerIterator &ver,
+				    bool follow_recommends,
+				    bool follow_suggests)
 {
   pkgDepCache::StateCache &state=(*this)[pkg];
   aptitude_state &estate=get_ext_state(pkg);
+  pkgCache::VerIterator candver=state.CandidateVerIter(*this);
+  pkgCache::VerIterator instver=state.InstVerIter(*this);
 
-  // If a package was garbage-collected but is not being marked, we
-  // should re-mark it.  (note: undo==NULL is actually correct;
+  // If a package was garbage-collected but is now being marked, we
+  // should re-select it.  (note: undo==NULL is actually correct;
   // mark_and_sweep *never* stores undo info....)
   if(state.Delete() && estate.remove_reason==unused)
     {
-      mark_install(pkg, false, false, NULL);
-      MarkKeep(pkg);
+      if(ver==candver)
+	mark_install(pkg, false, false, NULL);
+      else if(ver==pkg.CurrentVer())
+	MarkKeep(pkg);
+
+      instver=state.InstVerIter(*this);
     }
 
-  if(!((ver==pkg.CurrentVer() && PkgState[pkg->ID].Keep()) ||
-       (ver==state.InstVerIter(GetCache()) &&
-	state.Install())))
+  if(!(ver==instver && !instver.end()))
     return;
 
   if(estate.marked)
@@ -1107,20 +1149,22 @@ void aptitudeDepCache::mark_package(const PkgIterator &pkg,
 	{
 	  if(d->Type==pkgCache::Dep::Depends ||
 	     d->Type==pkgCache::Dep::PreDepends ||
-	     (aptcfg->FindB(PACKAGE "::Recommends-Important", true) &&
+	     (follow_recommends &&
 	      d->Type==pkgCache::Dep::Recommends) ||
-	     (aptcfg->FindB(PACKAGE "::Suggests-Important", false) &&
+	     (follow_suggests &&
 	      d->Type==pkgCache::Dep::Suggests))
 	    {
 	      // Try all versions of this package.
 	      for(VerIterator V=d.TargetPkg().VersionList(); !V.end(); ++V)
 		if(_system->VS->CheckDep(V.VerStr(), d->CompareOp, d.TargetVer()))
-		  mark_package(V.ParentPkg(), V);
+		  mark_package(V.ParentPkg(), V,
+			       follow_recommends, follow_suggests);
 
 	      // Now try virtual packages
 	      for(PrvIterator prv=d.TargetPkg().ProvidesList(); !prv.end(); ++prv)
 		if(_system->VS->CheckDep(prv.ProvideVersion(), d->CompareOp, d.TargetVer()))
-		  mark_package(prv.OwnerPkg(), prv.OwnerVer());
+		  mark_package(prv.OwnerPkg(), prv.OwnerVer(),
+			       follow_recommends, follow_suggests);
 	    }
 	}
     }
@@ -1134,6 +1178,8 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
   if(mark_and_sweep_in_progress)
     return;
 
+  begin_action_group();
+
   mark_and_sweep_in_progress=true;
 
   std::string matchterm=aptcfg->Find(PACKAGE "::Delete-Unused-Pattern", "");
@@ -1145,6 +1191,10 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
       package_states[p->ID].garbage=false;
     }
 
+  bool follow_recommends=aptcfg->FindB(PACKAGE "::Recommends-Important", true);
+  bool follow_suggests=aptcfg->FindB(PACKAGE "::Keep-Suggests", false) ||
+    aptcfg->FindB(PACKAGE "::Suggests-Important", false);
+
   for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
     {
       if(package_states[p->ID].install_reason==manual ||
@@ -1156,12 +1206,16 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 		       matcher->matches(p, PkgState[p->ID].InstVerIter(GetCache()))))))
 	{
 	  if(PkgState[p->ID].Keep() && !p.CurrentVer().end())
-	    mark_package(p, p.CurrentVer());
+	    mark_package(p, p.CurrentVer(),
+			 follow_recommends, follow_suggests);
 	  else if(PkgState[p->ID].Install())
-	    mark_package(p, PkgState[p->ID].InstVerIter(GetCache()));
+	    mark_package(p, PkgState[p->ID].InstVerIter(GetCache()),
+			 follow_recommends, follow_suggests);
 	}
     }
 
+  bool delete_unused=aptcfg->FindB(PACKAGE "::Delete-Unused", true);
+
   for(pkgCache::PkgIterator p=PkgBegin(); !p.end(); ++p)
     {
       StateCache &state=(*this)[p];
@@ -1172,16 +1226,66 @@ void aptitudeDepCache::mark_and_sweep(undo_group *undo)
 	  estate.garbage=true;
 
 	  // Be sure not to re-delete already deleted packages.
-	  if((!p.CurrentVer().end() || state.Install()) &&
-	     !state.Delete() &&
-	     aptcfg->FindB(PACKAGE "::Delete-Unused", true))
-	    mark_delete(p, false, true, undo);
+	  if(delete_unused && (!p.CurrentVer().end() || state.Install()) &&
+	     !state.Delete())
+	    {
+	      bool do_delete=true;
+
+	      // If the package is being upgraded, check if we're
+	      // losing a versioned dep.  If the dependency matches
+	      // the previous version and not the new version, keep
+	      // the package back instead of removing it.
+	      if(!p.CurrentVer().end() && state.Install())
+		{
+		  const char *vs=p.CurrentVer().VerStr();
+
+		  // Check direct revdeps only.  THIS ASSUMES NO
+		  // VERSIONED PROVIDES, but Debian probably won't
+		  // have them for ages if ever.
+		  for(pkgCache::DepIterator revdep=p.RevDependsList();
+		      !revdep.end(); ++revdep)
+		    {
+		      pkgCache::PkgIterator depender=revdep.ParentPkg();
+		      // Find which version of the depending package
+		      // will be installed.
+		      pkgCache::VerIterator instver=(*this)[depender].InstVerIter(*this);
+
+		      // Only pay attention to strong positive
+		      // dependencies whose parents will be installed.
+		      if(revdep.ParentVer()==instver &&
+			 (revdep->Type==pkgCache::Dep::Depends ||
+			  revdep->Type==pkgCache::Dep::PreDepends ||
+			  (revdep->Type==pkgCache::Dep::Recommends &&
+			   follow_recommends)))
+			{
+			  // If the previous version matched, cancel the
+			  // deletion.  (note that I assume that the new
+			  // version does NOT match; otherwise it would
+			  // not be unused!)
+			  if(_system->VS->CheckDep(vs,
+						   revdep->CompareOp,
+						   revdep.TargetVer()))
+			    {
+			      mark_keep(p, false, false, undo);
+			      do_delete=false;
+			      break;
+			    }
+			}
+		    }
+		}
+
+	      if(do_delete)
+		mark_delete(p, false, true, undo);
+	    }
 	}
     }
 
   mark_and_sweep_in_progress=false;
 
   delete matcher;
+
+  // end_action_group is written carefully so it works here:
+  end_action_group(undo);
 }
 
 aptitudeCacheFile::aptitudeCacheFile()
@@ -1275,7 +1379,7 @@ bool aptitudeDepCache::is_held(const PkgIterator &pkg)
 {
   aptitude_state state=get_ext_state(pkg);
 
-  pkgCache::VerIterator candver=(*apt_cache_file)[pkg].CandidateVerIter(*apt_cache_file);
+  pkgCache::VerIterator candver=(*this)[pkg].CandidateVerIter(*this);
 
   return !pkg.CurrentVer().end() &&
     (state.selection_state == pkgCache::State::Hold ||
