@@ -2,12 +2,8 @@
 //
 //  Copyright 2001 Daniel Burrows
 //
-//  A policy class that allows Recommends and Suggests to be treated as
-// "always important", "important for new installs", or "never important".
-//
-//  Defaults are "new installs" for recommends and "never" for suggests..
-// I suspect that whatever I do will upset someone, and these seem like
-// sa[fn]e choices.
+//  A policy class that allows Recommends to be treated as "always
+// important", "important for new installs", or "never important".
 
 #include "aptitudepolicy.h"
 #include "../aptitude.h"
@@ -29,8 +25,6 @@ bool aptitudePolicy::IsImportantDep(pkgCache::DepIterator dep)
 
   if(dep->Type==pkgCache::Dep::Recommends)
     return aptcfg->FindB(PACKAGE "::Recommends-Important", true);
-  else if(dep->Type==pkgCache::Dep::Suggests)
-    return aptcfg->FindB(PACKAGE "::Suggests-Important", false);
   else
     return false;
 }
