@@ -24,6 +24,13 @@
 //  MORE IMPORTANT NOTE: Altering the value of PACKAGE::Theme will
 // not trigger ANY signal emission right now.  (this *will* be a nuisance
 // and I hope to change it eventually)
+//
+//
+//
+//  In order to allow only non-default options to be saved, this does the
+// following (hack :P ) -- it stores TWO configuration dictionaries, updating
+// BOTH whenever an option is set, but only saving from the user's.  (in
+// fact, it uses three -- ouch!)
 
 #ifndef CONFIG_SIGNAL_H
 #define CONFIG_SIGNAL_H
@@ -33,6 +40,8 @@
 #include <sigc++/slot.h>
 #include <sigc++/object.h>
 
+#include <iosfwd>
+
 #ifdef HAVE_CONFIG_H
 #include "../../config.h"
 #endif
@@ -55,12 +64,9 @@
 
 class signalling_config:public SigC::Object
 {
-  // Having two configuration objects is a workaround for a bug in apt;
-  // the long configuration-item names were causing problems with the
-  // http fetcher.
-  Configuration *real_config, *theme_config;
+  Configuration *user_config, *system_config, *theme_config;
 
-  typedef HASH_NAMESPACE::hash_map<string, SigC::Signal1<void, string> *> connhash;
+  typedef HASH_NAMESPACE::hash_map<string, SigC::Signal0<void> *> connhash;
 
   connhash conn_table;
 
@@ -68,85 +74,102 @@ class signalling_config:public SigC::Object
   // this will be "PACKAGE::UI::Themes::bland::")
   std::string themeroot;
   void update_theme(std::string theme);
+  void do_update_theme();
+
 public:
-  signalling_config(Configuration *_real_config,
+  // Assumes that all settings initially contained in user_config are
+  // also contained in system_config.
+  signalling_config(Configuration *_user_config,
+		    Configuration *_system_config,
 		    Configuration *_theme_config);
   ~signalling_config();
 
   inline string Find(const char *Name,const char *Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->Find(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->Find(Name, Default);
     else
       return theme_config->Find(themeroot+Name, Default);
   }
   inline string Find(string Name,const char *Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->Find(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->Find(Name, Default);
     else
-      return theme_config->Find((themeroot+Name).c_str(), Default);
+      return theme_config->Find(themeroot+Name, Default);
   }
 
   inline string FindFile(const char *Name,const char *Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindFile(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindFile(Name, Default);
     else
       return theme_config->FindFile((themeroot+Name).c_str(), Default);
   }
 
   inline string FindDir(const char *Name,const char *Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindDir(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindDir(Name, Default);
     else
       return theme_config->FindDir((themeroot+Name).c_str(), Default);
   }
 
   inline int FindI(const char *Name,int Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindI(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindI(Name, Default);
     else
       return theme_config->FindI(themeroot+Name, Default);
   }
 
   inline int FindI(string Name,bool Default = 0)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindI(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindI(Name, Default);
     else
       return theme_config->FindI(themeroot+Name, Default);
   }
 
   inline bool FindB(const char *Name,bool Default = false)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindB(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindB(Name, Default);
     else
       return theme_config->FindB(themeroot+Name, Default);
   }
 
   inline bool FindB(string Name,bool Default = false)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->FindB(Name, Default);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->FindB(Name, Default);
     else
       return theme_config->FindB(themeroot+Name, Default);
   }
 
   void Set(string Name,string Value);
-  void Set(const char *Name,string Value);
-  void Set(const char *Name,int Value);
+  void Set(string Name,int Value);
 
-  Configuration *setcfg(Configuration *newcfg);
-  // Switches to a different configuration tree (this will involve emitting
-  // various signals)  Returns the old configuration tree.
+  /** Sets Name to Value without modifying the user's configuration. */
+  void SetNoUser(string Name, string Value);
+
+  /** Sets Name to Value without modifying the user's configuration. */
+  void SetNoUser(string name, int Value);
+
+  /** Switches to a different configuration tree (this will involve
+   *  emitting various signals)
+   *
+   *  This assumes that the "user" configuration tree has been
+   *  appropriately modified and that new_system_cfg has the
+   *  appropriate correspondence with it.
+   */
+  void setcfg(Configuration *new_user_cfg,
+	      Configuration *new_system_cfg,
+	      Configuration *new_theme_cfg);
 
   inline bool Exists(string Name)
   {
-    if(real_config->Exists(Name))
+    if(system_config->Exists(Name))
       return true;
     else
       return !themeroot.empty() && theme_config->Exists(themeroot+Name);
@@ -154,16 +177,21 @@ public:
 
   bool Exists(const char *Name)
   {
-    if(real_config->Exists(Name))
+    if(system_config->Exists(Name))
       return true;
     else
       return !themeroot.empty() && theme_config->Exists(themeroot+Name);
   }
-      
+
+  inline bool ExistsUser(string Name)
+  {
+    return user_config->Exists(Name);
+  }
+
   inline const Configuration::Item *Tree(const char *Name)
   {
-    if(themeroot.empty() || real_config->Exists(Name))
-      return real_config->Tree(Name);
+    if(themeroot.empty() || system_config->Exists(Name))
+      return system_config->Tree(Name);
     else
       return theme_config->Tree((themeroot+Name).c_str());
   }
@@ -172,15 +200,12 @@ public:
   //        I think.
   Configuration *get_cfg(bool theme)
   {
-    return theme?theme_config:real_config;
+    return theme?theme_config:system_config;
   }
 
-  void connect(string name, const SigC::Slot1<void, std::string> &slot);
-  void connectI(string name, SigC::Slot1<void, int> slot);
-  void connectB(string name, SigC::Slot1<void, bool> slot);
+  void connect(string name, const SigC::Slot0<void> &slot);
 
-  void Dump()
-  {real_config->Dump();}
+  void Dump(std::ostream &out);
 };
 
 #endif
