@@ -37,6 +37,7 @@
 #include "../aptitude.h"
 
 #include <apt-pkg/error.h>
+#include <apt-pkg/pkgcache.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/version.h>
 
@@ -136,13 +137,6 @@ bool maintainer_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::V
 }
 typedef pkg_string_matcher<maintainer_matches> pkg_maintainer_matcher;
 
-bool priority_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
-{
-  return !ver.end() && ver.PriorityType() &&
-    string_matches(pattern, ver.PriorityType());
-}
-typedef pkg_string_matcher<priority_matches> pkg_priority_matcher;
-
 bool section_matches(regex_t *pattern, pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
 {
   return !ver.end() && ver.Section() && string_matches(pattern, ver.Section());
@@ -241,6 +235,22 @@ public:
   }
 };
 
+class pkg_priority_matcher:public pkg_matcher
+{
+  pkgCache::State::VerPriority type;
+public:
+  pkg_priority_matcher(pkgCache::State::VerPriority _type)
+    :type(_type) {}
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    if(ver.end())
+      return false;
+    else
+      return ver->Priority == type;
+  }
+};
+
 // Matches packages with unmet dependencies of a particular type.
 class pkg_broken_type_matcher:public pkg_matcher
 {
@@ -260,7 +270,7 @@ public:
 	while(!dep.end())
 	  {
 	    // Skip to the end of the Or group to check GInstall
-	    while(dep->CompareOp & pkgDepCache::Dep::Or)
+	    while(dep->CompareOp & pkgCache::Dep::Or)
 	      ++dep;
 
 	    if(dep->Type==type &&
@@ -315,13 +325,12 @@ public:
 };
 
 class pkg_virtual_matcher:public pkg_matcher
-// Matches *pure* virtual packages -- ie, those that /have/ no versions (and
-// therefore will always pass in an 'end' version)
+// Matches *pure* virtual packages -- ie, those that /have/ no versions.
 {
 public:
   bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
   {
-    return ver.end();
+    return pkg.VersionList().end();
   }
 };
 
@@ -362,19 +371,15 @@ public:
 // FIXME: make this more general (for recommends, eg?)
 class pkg_dep_matcher:public pkg_matcher
 {
-public:
-  enum match_mode {depends, conflict};
-
 private:
   pkg_matcher *pattern;
-  match_mode mode;
+  pkgCache::Dep::DepType type;
 
 public:
-  pkg_dep_matcher(pkg_matcher *_pattern,
-		  match_mode _mode)
+  pkg_dep_matcher(pkgCache::Dep::DepType _type,
+		  pkg_matcher *_pattern)
+    :pattern(_pattern), type(_type)
   {
-    pattern=_pattern;
-    mode=_mode;
   }
   ~pkg_dep_matcher() {delete pattern;}
 
@@ -386,14 +391,21 @@ public:
 
     for(pkgCache::DepIterator dep=ver.DependsList(); !dep.end(); ++dep)
       {
-	if( (mode==depends && (dep->Type==pkgCache::Dep::Depends ||
-			       dep->Type==pkgCache::Dep::PreDepends)) ||
-	    (mode==conflict && (dep->Type==pkgCache::Dep::Conflicts)))
+	if( (type == dep->Type) ||
+	    (type==pkgCache::Dep::Depends &&
+	     dep->Type==pkgCache::Dep::PreDepends))
 	  {
+	    // See if a versionless match works,.
+	    if(dep.TargetPkg().VersionList().end() &&
+	       pattern->matches(dep.TargetPkg(), dep.TargetPkg().VersionList()))
+	      return true;
+
 	    for(pkgCache::VerIterator i=dep.TargetPkg().VersionList(); !i.end(); i++)
 	      if(_system->VS->CheckDep(i.VerStr(), dep->CompareOp, dep.TargetVer()))
-		if(pattern->matches(dep.TargetPkg(), i))
-		  return true;
+		{
+		  if(pattern->matches(dep.TargetPkg(), i))
+		    return true;
+		}
 	  }
       }
 
@@ -452,6 +464,138 @@ public:
   }
 };
 
+// A dummy matcher that matches any package.
+class pkg_true_matcher:public pkg_matcher
+{
+public:
+  pkg_true_matcher() {}
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    return true;
+  }
+};
+
+// A dummy matcher that matches no packages.
+class pkg_false_matcher:public pkg_matcher
+{
+public:
+  pkg_false_matcher() {}
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    return false;
+  }
+};
+
+// Matches packages which have a dependency of the given type declared
+// on them by a package matching a given pattern.
+//
+// Traces through Provided packages as well.
+class pkg_revdep_matcher:public pkg_matcher
+{
+  pkgCache::Dep::DepType type;
+  pkg_matcher *pattern;
+public:
+  pkg_revdep_matcher(pkgCache::Dep::DepType _type,
+		     pkg_matcher *_pattern):type(_type), pattern(_pattern)
+  {
+  }
+
+  ~pkg_revdep_matcher()
+  {
+    delete pattern;
+  }
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    // Check direct dependencies.
+    for(pkgCache::DepIterator d=pkg.RevDependsList(); !d.end(); ++d)
+      {
+	if((d->Type==type ||
+	    (type==pkgCache::Dep::Depends && d->Type==pkgCache::Dep::PreDepends)) &&
+	   (!d.TargetVer() || (!ver.end() &&
+			       _system->VS->CheckDep(ver.VerStr(), d->CompareOp, d.TargetVer()))) &&
+	   pattern->matches(d.ParentPkg(), d.ParentVer()))
+	  return true;
+      }
+
+    // Check dependencies through virtual packages.  ie, things that Depend
+    // on stuff this package [version] Provides.
+    if(!ver.end())
+      for(pkgCache::PrvIterator p=ver.ProvidesList(); !p.end(); ++p)
+	{
+	  for(pkgCache::DepIterator d=p.ParentPkg().RevDependsList();
+	      !d.end(); ++d)
+	    {
+	      // Only unversioned dependencies can match here.
+	      if(d->Type==type && !d.TargetVer() &&
+		 pattern->matches(d.ParentPkg(), d.ParentVer()))
+		return true;
+	    }
+	}
+
+    return false;
+  }
+};
+
+
+/** Matches packages that provide a package that matches the given
+ *  pattern.
+ */
+class pkg_provides_matcher:public pkg_matcher
+{
+  pkg_matcher *pattern;
+public:
+  pkg_provides_matcher(pkg_matcher *_pattern):pattern(_pattern) {}
+
+  ~pkg_provides_matcher() 
+  {
+    delete pattern;
+  }
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    if(ver.end())
+      return false;
+
+    for(pkgCache::PrvIterator p=ver.ProvidesList(); !p.end(); ++p)
+      {
+	// Assumes no provided version.
+	if(pattern->matches(p.ParentPkg(), pkgCache::VerIterator(*apt_cache_file)))
+	  return true;
+      }
+
+    return false;
+  }
+};
+
+/** Matches packages which are provided by a package that fits the
+ *  given pattern.
+ */
+class pkg_revprv_matcher:public pkg_matcher
+{
+  pkg_matcher *pattern;
+public:
+  pkg_revprv_matcher(pkg_matcher *_pattern):pattern(_pattern) {}
+
+  ~pkg_revprv_matcher() 
+  {
+    delete pattern;
+  }
+
+  bool matches(pkgCache::PkgIterator pkg, pkgCache::VerIterator ver)
+  {
+    for(pkgCache::PrvIterator p=pkg.ProvidesList(); !p.end(); ++p)
+      {
+	if(pattern->matches(p.OwnerPkg(), p.OwnerVer()))
+	  return true;
+      }
+
+    return false;
+  }
+};
+
 //  Now back from the dead..it seems some people were actually using it ;-)
 //
 // Matches (non-virtual) packages which no installed package declares
@@ -553,6 +697,8 @@ pkgCache::Dep::DepType parse_deptype(string s)
     return pkgCache::Dep::Suggests;
   else if(!strcasecmp(s.c_str(), "conflicts"))
     return pkgCache::Dep::Conflicts;
+  else if(!strcasecmp(s.c_str(), "replaces"))
+    return pkgCache::Dep::Replaces;
   else // ewww.
     return (pkgCache::Dep::DepType) -1;
 }
@@ -667,8 +813,6 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		  return new pkg_broken_matcher;
 		case 'g':
 		  return new pkg_garbage_matcher;
-		case 'r':
-		  return new pkg_norevdep_matcher;
 		case 'c':
 		  return new pkg_configfiles_matcher;
 		case 'i':
@@ -681,18 +825,74 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		  return new pkg_new_matcher;
 		case 'U':
 		  return new pkg_upgradable_matcher;
-		case 'D':
+		case 'P':
 		case 'C':
 		  {
 		    pkg_matcher *m=parse_atom(s, loc, flag_errors);
 
 		    if(!m)
 		      return NULL;
+		    
+		    switch(s[prevloc+1])
+		      {
+		      case 'C':
+			return new pkg_dep_matcher(pkgCache::Dep::Conflicts, m);
+		      case 'P':
+			return new pkg_provides_matcher(m);
+		      }
+		  }
+		case 'D':
+		case 'R':
+		  {
+		    bool do_provides=false;
+		    pkgCache::Dep::DepType type=pkgCache::Dep::Depends;
+		    string::size_type nextloc=loc;
 
-		    if(s[prevloc+1]=='D')
-		      return new pkg_dep_matcher(m, pkg_dep_matcher::depends);
-		    else
-		      return new pkg_dep_matcher(m, pkg_dep_matcher::conflict);
+		    while(nextloc<s.size() && isalpha(s[nextloc]))
+		      ++nextloc;
+
+		    if(nextloc<s.size() && s[nextloc]==':')
+		      {
+			string tname(s, loc, nextloc-loc);
+
+			loc=nextloc+1;
+
+			if(!strcasecmp(tname.c_str(), "provides"))
+			  do_provides=true;
+			else
+			  {
+			    type=parse_deptype(tname.c_str());
+
+			    if(type==-1)
+			      {
+				char buf[512];
+				snprintf(buf, 512, _("Unknown dependency type: %s"),
+					 tname.c_str());
+				if(flag_errors)
+				  _error->Error("%s", buf);
+				return NULL;
+			      }
+			  }
+		      }
+
+		    pkg_matcher *m=parse_atom(s, loc, flag_errors);
+
+		    if(!m)
+		      return NULL;
+
+		    switch(s[prevloc+1])
+		      {
+		      case 'D':
+			if(do_provides)
+			  return new pkg_provides_matcher(m);
+			else
+			  return new pkg_dep_matcher(type, m);
+		      case 'R':
+			if(do_provides)
+			  return new pkg_revprv_matcher(m);
+			else
+			  return new pkg_revdep_matcher(type, m);
+		      }
 		  }
 		default:
 		  substr=parse_substr(s, loc);
@@ -737,20 +937,11 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 		    case 'A':
 		      return pkg_archive_matcher::init(substr, flag_errors);
 		    case 'B':
-		    case 'R':
 		      {
 			pkgCache::Dep::DepType ptype=parse_deptype(substr);
 
 			if(ptype!=-1)
-			  {
-			    switch(s[prevloc+1])
-			      {
-			      case 'B':
-				return new pkg_broken_type_matcher(ptype);
-			      case 'R':
-				return new pkg_norevdep_type_matcher(ptype);
-			      }
-			  }
+			  return new pkg_broken_type_matcher(ptype);
 			else
 			  {
 			    char buf[512];
@@ -761,22 +952,67 @@ pkg_matcher *parse_atom(string s, unsigned int &loc,
 			    return NULL;
 			  }
 		      }
-		    case 'n':
-		      return pkg_name_matcher::init(substr, flag_errors);
 		    case 'd':
 		      return pkg_description_matcher::init(substr, flag_errors);
+		    case 'F':
+		      return new pkg_false_matcher;
 		    case 'm':
 		      return pkg_maintainer_matcher::init(substr, flag_errors);
+		    case 'n':
+		      return pkg_name_matcher::init(substr, flag_errors);
 		    case 'O':
 		      return pkg_origin_matcher::init(substr, flag_errors);
 		    case 'p':
-		      return pkg_priority_matcher::init(substr, flag_errors);
+		      {
+			pkgCache::State::VerPriority type;
+
+			const char *s=substr.c_str();
+
+			if(strcasecmp(s, "important") == 0 ||
+			   (apt_cache_file &&
+			    strcasecmp(s, (*apt_cache_file)->GetCache().Priority(pkgCache::State::Important)) == 0))
+			  type=pkgCache::State::Important;
+			else if(strcasecmp(s, "required") == 0 ||
+				(apt_cache_file &&
+				 strcasecmp(s, (*apt_cache_file)->GetCache().Priority(pkgCache::State::Required)) == 0))
+			  type=pkgCache::State::Required;
+			else if(strcasecmp(s, "standard") == 0 ||
+				(apt_cache_file &&
+				 strcasecmp(s, (*apt_cache_file)->GetCache().Priority(pkgCache::State::Standard)) == 0))
+			  type=pkgCache::State::Standard;
+			else if(strcasecmp(s, "optional") == 0 ||
+				(apt_cache_file &&
+				 strcasecmp(s, (*apt_cache_file)->GetCache().Priority(pkgCache::State::Optional)) == 0))
+			  type=pkgCache::State::Optional;
+			else if(strcasecmp(s, "extra") == 0 ||
+				(apt_cache_file &&
+				 strcasecmp(s, (*apt_cache_file)->GetCache().Priority(pkgCache::State::Extra)) == 0))
+			  type=pkgCache::State::Extra;
+			else
+			  {
+			    if(flag_errors)
+			      {
+				char buf[512];
+
+				snprintf(buf, 512, _("Unknown priority %s"),
+					 substr.c_str());
+
+				_error->Error("%s", buf);
+			      }
+
+			    return NULL;
+			  }
+
+			return new pkg_priority_matcher(type);
+		      }
 		    case 's':
 		      return pkg_section_matcher::init(substr, flag_errors);
-		    case 'V':
-		      return pkg_version_matcher::init(substr, flag_errors);
 		    case 't':
 		      return pkg_task_matcher::init(substr, flag_errors);
+		    case 'T':
+		      return new pkg_true_matcher;
+		    case 'V':
+		      return pkg_version_matcher::init(substr, flag_errors);
 		    default:
 		      {
 			char buf[512];
