@@ -25,21 +25,24 @@
 #include <string>
 
 #include <apt-pkg/acquire.h>
+#include <apt-pkg/pkgcache.h>
 
 class pkg_changelog
 // Mainly used to make it easy (transparent) to clean up the changelog when
 // you're done with it.
 {
-  string filename;
+  std::string filename;
 
-  pkg_changelog::pkg_changelog(string _filename);
-  friend pkg_changelog *get_changelog(string, pkgAcquireStatus *);
+  pkg_changelog::pkg_changelog(std::string _filename);
+  friend pkg_changelog *get_changelog(pkgCache::VerIterator,
+				      pkgAcquireStatus *);
 public:
-  string get_filename() {return filename;}
+  std::string get_filename() {return filename;}
 
   ~pkg_changelog();
 };
 
-pkg_changelog *get_changelog(string package, pkgAcquireStatus *status);
+pkg_changelog *get_changelog(pkgCache::VerIterator ver,
+			     pkgAcquireStatus *status);
 
 #endif
