@@ -26,11 +26,11 @@
 #include <string>
 
 #if defined(HAVE_HASH_MAP) || defined(HAVE_EXT_HASH_MAP)
-namespace std
+namespace HASH_NAMESPACE
 {
-  struct hash<string>
+  struct hash<std::string>
   {
-    inline size_t operator()(const string &s) const
+    inline size_t operator()(const std::string &s) const
     {
       return hash<char *>()(s.c_str());
     }
