@@ -158,6 +158,98 @@ bool task::keys_present()
   return true;
 }
 
+static string rfc822_process_paragraph(const string &textdomain,
+				       string par)
+{
+  if (par.empty())
+    return par;
+
+  // Remove trailing whitespace
+  string::size_type loc = par.size()-1;
+  while(isspace(par[loc]))
+    --loc;
+  par.erase(loc+1, string::npos);
+
+  return dgettext(textdomain.c_str(), par.c_str());
+}
+
+/** Returns msgid translated in the given text domain, with
+ *  appropriate munging: paragraphs are translated individually after
+ *  one leading and all trailing whitespace on each line is stripped.
+ *
+ *  \param textdomain the domain in which to translate
+ *  \param msgid the formatted description which should be translated
+ */
+static string rfc822dgettext(string textdomain, string msgid)
+{
+  if (textdomain.empty())
+    return msgid;
+  string::size_type start=0, len=msgid.size(), nextnl=0;
+
+  string thispar = "";
+  string msgstr = "";
+
+  // Remove leading whitespaces, i.e. replace "\n " by '\n'
+  //
+  // This assumes the trailing whitespace exists already.
+  do
+    {
+      if (nextnl<len)
+	nextnl=msgid.find('\n', start);
+      if (nextnl==string::npos)
+	nextnl=len-1;
+      string thisline(msgid, start+1, nextnl-start);
+      thispar+=thisline;
+      start=nextnl+1;
+    }
+  while (start<len);
+
+  // Reformat text (replace '\n' by ' ') and translate individual
+  // paragraphs
+  bool verbatimline = (thispar[0] == ' ');
+  string::size_type loc=0;
+  start=0, len=thispar.size(), nextnl=0;
+  while((nextnl=thispar.find('\n', loc))!=string::npos)
+    {
+      // Verbatim line case
+      if (thispar[nextnl+1] == ' ')
+	verbatimline = true;
+      // End of the paragraph
+      else if (thispar[nextnl+1] == '.' && thispar[nextnl+2] == '\n')
+	{
+	  /* Translate current paragraph */
+	  msgstr+=rfc822_process_paragraph(textdomain, string(thispar, start, nextnl-start));
+	  msgstr+="\n.\n";
+	  start = nextnl + 3;
+	  nextnl += 2;
+	  verbatimline = false;
+	}
+      // Add to the paragraph
+      else
+	{
+	  if (!verbatimline)
+	    thispar[nextnl] = ' ';
+	  verbatimline = false;
+	}
+      loc = nextnl + 1;
+    }
+  msgstr+=rfc822_process_paragraph(textdomain, string(thispar, start));
+  // Remove trailing whitespace
+  loc=msgstr.size()-1;
+  while(isspace(msgstr[loc]))
+    --loc;
+  ++loc;
+  msgstr[loc] = '\0';
+  // Reformat text, i.e. replace '\n' by "\n "
+  start = 0;
+  while((nextnl=msgstr.find('\n', start))!=string::npos)
+    {
+      msgstr.insert(nextnl+1, " ");
+      start = nextnl+2;
+    }
+  return msgstr;
+}
+
 void load_tasks(OpProgress &progress)
 {
   // Build a list for each package of the tasks that package belongs to.
@@ -210,6 +302,7 @@ void load_tasks(OpProgress &progress)
 
   pkgTagFile tagfile(&task_file);
   pkgTagSection section;
+  string taskdomain="debian-tasks";
 
   while(tagfile.Step(section))
     {
@@ -221,6 +314,8 @@ void load_tasks(OpProgress &progress)
 	{
 	  istringstream keystr(section.FindS("Key"));
 
+	  keystr >> ws;
+
 	  while(!keystr.eof())
 	    {
 	      string s;
@@ -237,8 +332,9 @@ void load_tasks(OpProgress &progress)
 	  desc=section.FindS("Description");
 
 	  string::size_type newline=desc.find('\n');
-	  newtask.shortdesc=string(desc, 0, newline);
-	  newtask.longdesc=string(desc, newline);
+	  newtask.shortdesc=dgettext(taskdomain.c_str(), string(desc, 0, newline).c_str());
+	  newtask.longdesc=string("\n ");
+	  newtask.longdesc+=rfc822dgettext(taskdomain, string(desc, newline+1));
 
 	  (*task_list)[taskname]=newtask;
 	}
