@@ -10,11 +10,25 @@
 #include <map>
 #include <apt-pkg/pkgcache.h>
 
-struct task
+class OpProgress;
+
+class task
 {
+private:
+  bool keys_present_cache;
+  bool keys_present_cache_stale;
+public:
+  task():keys_present_cache(false), keys_present_cache_stale(true) {}
+
+  std::string name;
   std::string section;
   std::string shortdesc;
   std::string longdesc;
+  std::list<std::string> keys;
+
+  bool keys_present();
+
+  int relevance;
 };
 
 // The caller should **NOT** delete the list.
@@ -25,7 +39,7 @@ extern std::map<std::string, task> *task_list;
 
 // (re)loads in the current list of available tasks.  Necessary after a
 // cache reload, for obvious reasons.  apt_reload_cache will call this.
-void load_tasks();
+void load_tasks(OpProgress &progress);
 
 // Discards the current task list and readies a new one to be loaded.
 // Since the task list contains package iterators, we have to do something
