@@ -47,9 +47,11 @@ void load_colors(std::string group, bool use_theme)
 	    }
 
 	  if(!parse_color(fgstr, fg))
-	    _error->Error(_("Can't parse foreground of color \"%s\""), i->Tag.c_str());
+	    _error->Error(_("Can't parse foreground \"%s\"of color \"%s\""),
+			  fgstr.c_str(), i->Tag.c_str());
 	  else if(!parse_color(i->Child->Next->Value, bg))
-	    _error->Error(_("Can't parse background of color \"%s\""), i->Tag.c_str());
+	    _error->Error(_("Can't parse background \"%s\" of color \"%s\""),
+			  i->Child->Next->Value.c_str(), i->Tag.c_str());
 	  else
 	    set_color(i->Tag, fg, bg, attr);
 	}
