@@ -26,7 +26,6 @@
 #include <getopt.h>
 
 #include "aptitude.h"
-#include "cmdline.h"
 
 #include "../config.h"
 
@@ -37,6 +36,17 @@
 #include <vscreen/vscreen.h>
 #include <vscreen/vs_util.h>
 
+#include <cmdline/cmdline_changelog.h>
+#include <cmdline/cmdline_clean.h>
+#include <cmdline/cmdline_do_action.h>
+#include <cmdline/cmdline_download.h>
+#include <cmdline/cmdline_forget_new.h>
+#include <cmdline/cmdline_moo.h>
+#include <cmdline/cmdline_search.h>
+#include <cmdline/cmdline_show.h>
+#include <cmdline/cmdline_update.h>
+#include <cmdline/cmdline_upgrade.h>
+
 #include <sigc++/slot.h>
 
 #include <apt-pkg/init.h>
@@ -96,6 +106,7 @@ static void usage()
   printf(_(" show         - Display detailed information about a package\n"));
   printf(_(" clean        - Erase downloaded package files\n"));
   printf(_(" autoclean    - Erase old downloaded package files\n"));
+  printf(_(" changelog    - View a package's changelog\n"));
   printf(_(" download     - Download the .deb file for a package\n"));
   printf("\n");
   printf(_("  Options:\n"));
@@ -113,7 +124,8 @@ static void usage()
   printf(_(" -Z		    Show the change in installed size of each package.\n"));
   printf(_(" -v             Display extra information. (may be supplied multiple times)\n"));
   printf(_(" -t [release]   Set the release from which packages should be installed\n"));
-  printf(_(" --with(out)-recommends, --with(out)-suggests\n                Specify whether or not to treat recommends (suggests) as\n                strong dependencies\n"));
+  printf(_(" -o key=val     Directly set the configuration option named 'key'\n"));
+  printf(_(" --with(out)-recommends	Specify whether or not to treat recommends as\n                strong dependencies\n"));
   printf(_(" -S fname: Read the aptitude extended status info from fname.\n"));
   printf(_(" -u      : Download new package lists on startup.\n"));
   printf(_(" -i      : Perform an install run on startup.\n"));
@@ -122,7 +134,7 @@ static void usage()
 }
 
 // This handles options with no single-character equivalent
-#define OPTION_VERSION 1
+enum {OPTION_VERSION=1, OPTION_VISUAL_PREVIEW};
 int getopt_result;
 
 option opts[]={
@@ -133,8 +145,6 @@ option opts[]={
   {"simulate", 0, NULL, 's'},
   {"with-recommends", 0, NULL, 'r'},
   {"without-recommends", 0, NULL, 'R'},
-  {"with-suggests", 0, NULL, 'g'},
-  {"without-suggests", 0, NULL, 'G'},
   {"download-only", 0, NULL, 'd'},
   {"assume-yes", 0, NULL, 'y'},
   {"verbose", 0, NULL, 'v'},
@@ -143,6 +153,7 @@ option opts[]={
   {"prompt", 0, NULL, 'P'},
   {"sort", 1, NULL, 'O'},
   {"target-release", 1, NULL, 't'},
+  {"visual-preview", 0, &getopt_result, OPTION_VISUAL_PREVIEW},
   {0,0,0,0}
 };
 
@@ -172,6 +183,7 @@ int main(int argc, char *argv[])
   bool showvers=aptcfg->FindB(PACKAGE "::CmdLine::Show-Versions", false);
   bool showdeps=aptcfg->FindB(PACKAGE "::CmdLine::Show-Deps", false);
   bool showsize=aptcfg->FindB(PACKAGE "::CmdLine::Show-Size-Changes", false);
+  bool visual_preview=aptcfg->FindB(PACKAGE "::CmdLine::Visual-Preview", false);
   bool always_prompt=aptcfg->FindB(PACKAGE "::CmdLine::Always-Prompt", false);
   int verbose=aptcfg->FindI(PACKAGE "::CmdLine::Verbose", 0);
 
@@ -182,8 +194,11 @@ int main(int argc, char *argv[])
   //      rocker.
   argv0=argv[0];
 
+  // This will be used to hold current locale.
+  char *locale;
+
   // Read the arguments:
-  while((curopt=getopt_long(argc, argv, "DVZvhS:uiF:w:sO:fdyPt:", opts, NULL))!=-1)
+  while((curopt=getopt_long(argc, argv, "DVZvhS:uiF:w:sO:fdyPt:Rro:", opts, NULL))!=-1)
     {
       switch(curopt)
 	{
@@ -221,19 +236,29 @@ int main(int argc, char *argv[])
 	  assume_yes=true;
 	  break;
 	case 'r':
-	  aptcfg->Set(PACKAGE "::Recommends-Important", "true");
+	  aptcfg->SetNoUser(PACKAGE "::Recommends-Important", "true");
 	  break;
 	case 'R':
-	  aptcfg->Set(PACKAGE "::Recommends-Important", "false");
-	  break;
-	case 'g':
-	  aptcfg->Set(PACKAGE "::Suggests-Important", "true");
-	  break;
-	case 'G':
-	  aptcfg->Set(PACKAGE "::Suggests-Important", "false");
+	  aptcfg->SetNoUser(PACKAGE "::Recommends-Important", "false");
 	  break;
 	case 't':
-	  aptcfg->Set("APT::Default-Release", optarg);
+	  aptcfg->SetNoUser("APT::Default-Release", optarg);
+	  break;
+	case 'o':
+	  {
+	    string s=optarg;
+	    string::size_type eqloc=s.find('=');
+
+	    // note that if eqloc!=s.npos, s.size()>0
+	    if(eqloc==s.npos || eqloc==s.size()-1)
+	      fprintf(stderr, _("-o requires an argument of the form key=value, got %s"), optarg);
+	    else
+	      {
+		string key(s, 0, eqloc), value(s, eqloc+1);
+
+		aptcfg->SetNoUser(key, value);
+	      }
+	  }
 	  break;
 	case 'v':
 	  ++verbose;
@@ -260,6 +285,16 @@ int main(int argc, char *argv[])
 	    case OPTION_VERSION:
 	      show_version();
 	      exit(0);
+	    case OPTION_VISUAL_PREVIEW:
+	      visual_preview=true;
+	      locale = setlocale(LC_ALL, NULL);    
+	      
+              if (strstr(locale, "UTF-8")) {
+	      	      setlocale(LC_ALL, "C");
+		      locale = setlocale(LC_ALL, NULL);
+              }
+	      
+	      break;
 	    default:
 	      fprintf(stderr, "%s",
 		      _("WEIRDNESS: unknown option code received\n"));
@@ -328,19 +363,21 @@ int main(int argc, char *argv[])
 	       (!strcasecmp(argv[optind], "markauto")) ||
 	       (!strcasecmp(argv[optind], "unmarkauto")) ||
 	       (!strcasecmp(argv[optind], "forbid-version")) )
-	return cmdline_pkgaction(argc-optind, argv+optind,
+	return cmdline_do_action(argc-optind, argv+optind,
 				 status_fname,
 				 simulate, assume_yes, download_only,
 				 fix_broken, showvers, showdeps, showsize,
-				 always_prompt, verbose);
+				 visual_preview, always_prompt, verbose);
       else if(!strcasecmp(argv[optind], "upgrade"))
 	return cmdline_upgrade(argc-optind, argv+optind,
 			       status_fname,
 			       simulate, assume_yes, download_only,
 			       showvers, showdeps, showsize,
-			       always_prompt, verbose);
+			       visual_preview, always_prompt, verbose);
       else if(!strcasecmp(argv[optind], "download"))
 	return cmdline_download(argc-optind, argv+optind);
+      else if(!strcasecmp(argv[optind], "changelog"))
+	return cmdline_changelog(argc-optind, argv+optind);
       else if(!strcasecmp(argv[optind], "moo"))
 	return cmdline_moo(argc-optind, argv+optind, verbose);
       else if(!strcasecmp(argv[optind], "show"))
@@ -365,6 +402,15 @@ int main(int argc, char *argv[])
       exit(1);
     }
 
+  // Apart from visual preview we also have to take care of the normal
+  // ui mess with UTF-8, and fallback to "C" locale
+  locale = setlocale(LC_ALL, NULL);    
+	      	      
+  if (strstr(locale, "UTF-8")) {
+    setlocale(LC_ALL, "C");
+    locale = setlocale(LC_ALL, NULL);
+  }			    	
+  
   ui_init();
 
   vs_progress *p=gen_progress_bar();
