@@ -180,8 +180,10 @@ void cmine::do_start_custom_game(vscreen_widget *w,
 
   long height=strtol(s.c_str(), &end, 0);
 
-  if(s.c_str()[0]=='\0' || *end!='\0')
+  if(s.c_str()[0]=='\0' || *end!='\0' || height<1)
     {
+      // FIXME: should say "positive integer" but the translators will
+      // lynch me ;-)
       popup_widget(vs_dialog_ok(_("The board height must be a number"),
 				NULL,
 				get_color("Error")));
@@ -192,7 +194,7 @@ void cmine::do_start_custom_game(vscreen_widget *w,
   end=const_cast<char *>(s.c_str());
   long width=strtol(s.c_str(), &end, 0);
 
-  if(s.c_str()[0]=='\0' || *end!='\0')
+  if(s.c_str()[0]=='\0' || *end!='\0' || width<1)
     {
       popup_widget(vs_dialog_ok(_("The board width must be a number"),
 				NULL,
@@ -204,7 +206,7 @@ void cmine::do_start_custom_game(vscreen_widget *w,
   end=const_cast<char *>(s.c_str());
   long mines=strtol(s.c_str(), &end, 0);
 
-  if(s.c_str()[0]=='\0' || *end!='\0')
+  if(s.c_str()[0]=='\0' || *end!='\0' || mines<1)
     {
       // awkward wording
       popup_widget(vs_dialog_ok(_("The number of mines must be a number"),
@@ -385,7 +387,7 @@ void cmine::do_continue_new_game(bool start,
   w->destroy();
 };
 
-cmine::cmine():board(NULL)
+cmine::cmine():board(NULL), prevwidth(0), prevheight(0)
 {
   set_board(easy_game());
   vscreen_addtimeout(slot(*this, &cmine::update_header), 500);
@@ -649,14 +651,14 @@ void cmine::paint_square(int x, int y)
 	      if(entry.flagged)
 		ch='F'|get_color("MineFlagColor");
 	      else
-		ch=' '|get_color("ScreenBackgroundColor");
+		ch=' '|get_color("DefaultWidgetBackground");
 	    }
 	  else if(entry.contains_mine)
 	    ch='^'|get_color("MineBombColor");
 	  else if(entry.adjacent_mines==0)
-	    ch='.'|get_color("ScreenBackgroundColor");
+	    ch='.'|get_color("DefaultWidgetBackground");
 	  else
-	    ch=('0'+entry.adjacent_mines)|get_color("ScreenBackgroundColor");
+	    ch=('0'+entry.adjacent_mines)|get_color("DefaultWidgetBackground");
 	}
       else
 	{
@@ -671,12 +673,12 @@ void cmine::paint_square(int x, int y)
 	  else if(entry.uncovered)
 	    {
 	      if(entry.adjacent_mines==0)
-		ch='.'|get_color("ScreenBackgroundColor");
+		ch='.'|get_color("DefaultWidgetBackground");
 	      else
-		ch=('0'+entry.adjacent_mines)|get_color("ScreenBackgroundColor");
+		ch=('0'+entry.adjacent_mines)|get_color("DefaultWidgetBackground");
 	    }
 	  else
-	    ch=' '|get_color("ScreenBackgroundColor");
+	    ch=' '|get_color("DefaultWidgetBackground");
 	}
       if(board->get_state()==mine_board::playing && x==curx && y==cury)
 	ch|=A_REVERSE;
@@ -712,7 +714,7 @@ void cmine::paint()
 
       if(board)
 	{
-	  attrset(get_color("ScreenBackgroundColor"));
+	  attrset(get_color("DefaultWidgetBackground"));
 
 	  int right=basex+board->get_width()+1, down=basey+1+board->get_height()+1;
 	  // x and y coordinates, respectively, of the right and lower board
@@ -812,8 +814,8 @@ void cmine::init_bindings()
   global_bindings.set("MineUncoverSweepSquare", KEY_ENTER);
   global_bindings.set("MineFlagSquare", 'f');
   global_bindings.set("MineNewGame", 'n');
-  global_bindings.set("MineSaveGame", 's');
-  global_bindings.set("MineLoadGame", 'l');
+  global_bindings.set("MineSaveGame", 'S');
+  global_bindings.set("MineLoadGame", 'L');
 
   bindings=new keybindings(&global_bindings);
 }
