@@ -2,7 +2,7 @@
 //
 //  The pkg_columnizer class.
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999,2000,2001, 2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <generic/tasks.h>
 
 #include <apt-pkg/error.h>
+#include <apt-pkg/init.h> // For the APT version.
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/policy.h>
 #include <apt-pkg/strutl.h>
@@ -36,7 +37,11 @@
 #include <unistd.h>
 
 column_definition_list *pkg_item::pkg_columnizer::columns=NULL;
+#if APT_PKG_MAJOR >=3 && APT_PKG_MINOR >= 5
+const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M%S %p %Z %v %V";
+#else
 const char *pkg_item::pkg_columnizer::default_pkgdisplay="%c%a%M %p %Z %v %V";
+#endif
 
 // NOTE: the default widths here will be overridden by the initialization
 //      code.
@@ -66,7 +71,10 @@ column_type_defaults pkg_item::pkg_columnizer::defaults[pkg_columnizer::numtypes
   {30, false, true},    // diskusage
   {15, false, true},    // downloadsize
   {4, false, false},    // pin_priority
-  {15, false, true}
+  {15, false, true},    // hostname
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+  {1, false, false},    // trust_state
+#endif
 };
 
 // Default widths for:
@@ -510,6 +518,17 @@ column_disposition pkg_item::pkg_columnizer::setup_column(int type)
 	return column_disposition(buf, 0);
       }
       break;
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+    case trust_state:
+      {
+	if(visible_ver.end())
+	  return column_disposition(" ", 0);
+	else if(package_trusted(visible_ver))
+	  return column_disposition(" ", 0);
+	else
+	  return column_disposition("U", 0);
+      }
+#endif
     default:
       return column_disposition(_("ERROR"), 0);
     }
@@ -546,6 +565,10 @@ int pkg_item::pkg_columnizer::parse_column_type(char id)
       return priority;
     case 'R':
       return shortpriority;
+#if APT_PKG_MAJOR >=3 && APT_PKG_MINOR >= 5
+    case 'S':
+      return trust_state;
+#endif
     case 's':
       return section;
     case 'r':
