@@ -1,6 +1,6 @@
 // pkg_columnizer.h  -*-c++-*-
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999-2002, 2004-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #include "pkg_item.h"
 #include "vscreen/config/column_definition.h"
 
+#include <apt-pkg/init.h> // For the APT version.
+
 class pkg_item::pkg_columnizer:public column_generator
 {
   class pkg_genheaders;
@@ -57,7 +59,12 @@ public:
 
               progname, progver, brokencount, diskusage, downloadsize,
 	      pin_priority,
-              hostname, numtypes};
+              hostname,
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+	      trust_state, numtypes};
+#else
+	      numtypes};
+#endif
   static column_type_defaults defaults[numtypes];
   static const char *column_names[numtypes];
 
