@@ -170,7 +170,7 @@ void pkg_grouppolicy_section::init_section_descriptions()
   section_descriptions["libdevel"]=_("Development files for libraries\n Packages in the 'libdevel' section contain files required for building programs that use libraries in the 'libs' section.  You don't need packages from this section unless you want to compile software yourself.");
   section_descriptions["libs"]=_("Collections of software routines\n Packages in the 'libs' section provide necessary functionality for other software on the computer. With very few exceptions, you should not need to explicitly install a package from this section; the package system will install them as required to fulfill dependencies.");
   section_descriptions["perl"]=_("Perl interpreter and libraries\n Packages in the 'perl' section provide the Perl programming language and many third-party libraries for it. Unless you are a Perl programmer, you don't need to install packages from this section explicitly; the package system will install them if they are required.");
-  section_descriptions["python"]=_("Python interpreter and libraries\n Packages in the 'python' section provide the Python programming language and many third-party libraries for it. Unless you are a Perl programmer, you don't need to install packages from this section explicitly; the package system will install them if they are required.");
+  section_descriptions["python"]=_("Python interpreter and libraries\n Packages in the 'python' section provide the Python programming language and many third-party libraries for it. Unless you are a Python programmer, you don't need to install packages from this section explicitly; the package system will install them if they are required.");
   section_descriptions["mail"]=_("Programs to write, send, and route email messages\n Packages in the 'mail' section include mail readers, mail transport daemons, mailing list software, and spam filters, as well as various other software related to electronic mail.");
   section_descriptions["math"]=_("Numeric analysis and other mathematics-related software\n Packages in the 'math' section include calculators, languages for mathematical computation (similar to Mathematica), symbolic algebra packages, and programs to visualize mathematical objects.");
   section_descriptions["misc"]=_("Miscellaneous software\n Packages in the 'misc' section have too unusual a function to be classified.");
@@ -422,43 +422,12 @@ bool pkg_missing_filter(const pkgCache::PkgIterator &pkg)
 
 /*****************************************************************************/
 
-/** Returns true if pkg is suggested by another package which will be
- * installed.
- */
-static bool package_suggested(const pkgCache::PkgIterator &pkg)
-{
-  pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
-  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
-
-  for(pkgCache::DepIterator d=pkg.RevDependsList(); !d.end(); ++d)
-    if(d->Type==pkgCache::Dep::Suggests)
-      {
-	// Check whether the package doing the depending is going
-	// to be installed.
-	pkgCache::PkgIterator depender=d.ParentPkg();
-	pkgDepCache::StateCache &depstate=(*apt_cache_file)[depender];
-	pkgCache::VerIterator depinstver=depstate.InstVerIter(*apt_cache_file);
-
-	if(depender.CurrentVer().end() &&
-	   depstate.Install() &&
-	   !depinstver.end() &&
-	   !candver.end() &&
-	   _system->VS->CheckDep(candver.VerStr(),
-				 d->CompareOp, d.TargetVer()))
-	  {
-	    if(d.ParentVer()==depinstver)
-	      return true;
-	  }
-      }
-
-  return false;
-}
-
 class pkg_grouppolicy_mode:public pkg_grouppolicy
 {
   const static char * const child_names[];
   pair<pkg_grouppolicy *, pkg_subtree *> children[num_pkg_action_states];
   pair<pkg_grouppolicy *, pkg_subtree *> suggested_child;
+  pair<pkg_grouppolicy *, pkg_subtree *> recommended_child;
   pkg_grouppolicy_factory *chain;
 public:
   pkg_grouppolicy_mode(pkg_grouppolicy_factory *_chain,
@@ -495,6 +464,26 @@ public:
 
 	children[group].first->add_package(pkg, children[group].second);
       }
+    else if(pkg.CurrentVer().end() && package_recommended(pkg))
+      {
+	if(!recommended_child.second)
+	  {
+	    string desc=_("Packages which are recommended by other packages\n These packages are not strictly required, but they may be necessary to provide full functionality in some other programs that you are installing or upgrading.");
+	    string shortdesc=string(desc, 0, desc.find('\n'));
+
+	    pkg_subtree *newtree=new pkg_subtree_with_order(shortdesc,
+							    desc,
+							    get_desc_sig(),
+							    num_pkg_action_states,
+							    true);
+	    root->add_child(newtree);
+	    recommended_child.first=chain->instantiate(get_sig(),
+						       get_desc_sig());
+	    recommended_child.second=newtree;
+	  }
+
+	recommended_child.first->add_package(pkg, recommended_child.second);
+      }
     else if(pkg.CurrentVer().end() && package_suggested(pkg))
       {
 	if(!suggested_child.second)
@@ -505,7 +494,7 @@ public:
 	    pkg_subtree *newtree=new pkg_subtree_with_order(shortdesc,
 							    desc,
 							    get_desc_sig(),
-							    num_pkg_action_states,
+							    num_pkg_action_states+1,
 							    false);
 	    root->add_child(newtree);
 	    suggested_child.first=chain->instantiate(get_sig(),
@@ -888,13 +877,13 @@ void pkg_grouppolicy_task::init_section_descriptions()
   if(already_done)
     return;
 
-  task_section_descriptions.push_back(pair<string,string>("user", "End-user"));
-  task_section_descriptions.push_back(pair<string,string>("server", "Servers"));
-  task_section_descriptions.push_back(pair<string,string>("devel", "Development"));
-  task_section_descriptions.push_back(pair<string,string>("l10n", "Localization"));
-  task_section_descriptions.push_back(pair<string,string>("hware", "Hardware Support"));
-  task_section_descriptions.push_back(pair<string,string>("misc", "Miscellaneous"));
-  task_section_descriptions.push_back(pair<string,string>("unknown", "Unrecognized tasks"));
+  task_section_descriptions.push_back(pair<string,string>("user", dgettext("tasksel", "End-user")));
+  task_section_descriptions.push_back(pair<string,string>("server", dgettext("tasksel", "Servers")));
+  task_section_descriptions.push_back(pair<string,string>("devel", dgettext("tasksel", "Development")));
+  task_section_descriptions.push_back(pair<string,string>("l10n", dgettext("tasksel", "Localization")));
+  task_section_descriptions.push_back(pair<string,string>("hware", dgettext("tasksel", "Hardware Support")));
+  task_section_descriptions.push_back(pair<string,string>("misc", dgettext("tasksel", "Miscellaneous")));
+  task_section_descriptions.push_back(pair<string,string>("unknown", dgettext("tasksel", "Unrecognized tasks")));
 }
 
 /** Uses the fact that 0<=Relevance<=10 to encode Relevance as a character. */
