@@ -48,7 +48,7 @@
 #define PKG_GROUPPOLICY_H
 
 #include <apt-pkg/pkgcache.h>
-#include <sigc++/basic_signal.h>
+#include <sigc++/signal.h>
 
 // Used to set the visible status if a package is available.
 typedef SigC::Signal2<void,
