@@ -1,6 +1,6 @@
 // pkg_info_screen.cc
 //
-//  Copyright 2000 Daniel Burrows
+//  Copyright 2000-2002, 2004-2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -34,9 +34,11 @@
 #include "pkg_item_with_subtree.h"
 #include "dep_item.h"
 #include "pkg_ver_item.h"
+#include "trust.h"
 
 #include "generic/apt.h"
 
+#include <apt-pkg/init.h> // For the APT version.
 #include <apt-pkg/strutl.h>
 
 class pkg_grouppolicy_info:public pkg_grouppolicy
@@ -83,6 +85,15 @@ void pkg_grouppolicy_info::setup_package_info(const pkgCache::PkgIterator &pkg,
 
       vector<fragment*> frags;
 
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+      fragment *untrusted_warning=make_untrusted_warning(ver);
+      if(untrusted_warning != NULL)
+	{
+	  frags.push_back(untrusted_warning);
+	  frags.push_back(newline_fragment());
+	}
+#endif
+
       // Avoid creating new strings to translate.
       frags.push_back(clipbox(fragf("%B%s%b%s%n",
 				    _("Description: "), shortdesc.c_str())));
