@@ -25,23 +25,25 @@
 #define PKG_INFO_SCREEN_H
 
 #include "pkg_grouppolicy.h"
-#include "vs_tree.h"
+
+#include "apt_info_tree.h"
 
 #include <apt-pkg/cacheiterators.h>
+#include <string>
 
 class pkg_grouppolicy_info_factory:public pkg_grouppolicy_factory
 {
 public:
-  pkg_grouppolicy *instantiate(pkg_subtree *root);
+  pkg_grouppolicy *instantiate();
 };
 
-class pkg_info_screen:public vs_tree
+class pkg_info_screen:public apt_info_tree
 {
-  vscreen *prev;
+protected:
+  vs_treeitem *setup_new_root(const pkgCache::PkgIterator &pkg,
+			      const pkgCache::VerIterator &ver);
 public:
-  pkg_info_screen(pkgCache::PkgIterator pkg);
-
-  void dispatch_char(chtype ch);
+  pkg_info_screen(const pkgCache::PkgIterator &pkg, const pkgCache::VerIterator &ver);
 };
 
 #endif
