@@ -34,6 +34,7 @@
 #include "vs_progress.h"
 
 #include <generic/apt.h>
+#include <generic/apt_undo_group.h>
 #include <generic/config_signal.h>
 #include <generic/matchers.h>
 
@@ -145,6 +146,15 @@ void pkg_item::hold(undo_group *undo)
 				 undo);
 }
 
+void pkg_item::keep(undo_group *undo)
+{
+  // Keep, don't hold, the package.
+  (*apt_cache_file)->mark_keep(package,
+			       false,
+			       (*apt_cache_file)->get_ext_state(package).selection_state==pkgCache::State::Hold,
+			       undo);
+}
+
 void pkg_item::do_remove(undo_group *undo)
 {
   if((package->Flags&pkgCache::Flag::Essential)==pkgCache::Flag::Essential ||
@@ -202,6 +212,21 @@ void pkg_item::set_auto(bool isauto, undo_group *undo)
   (*apt_cache_file)->mark_auto_installed(package, isauto, undo);
 }
 
+void pkg_item::show_information()
+{
+  char buf[512]="Foo";
+  snprintf(buf, 512, _("Information about %s"), package.Name());
+
+  vscreen_widget *w=make_info_screen(package, visible_version());
+  // what to use as the menu description?
+  insert_main_widget(w, buf, "");
+}
+
+void pkg_item::show_changelog()
+{
+   view_changelog(visible_version());
+}
+
 int pkg_item::get_normal_attr()
 {
   return pkg_attr(package, vs_treeitem::get_normal_attr());
@@ -299,16 +324,10 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       insert_main_widget(w, buf, "");
     }
   else if(bindings->key_matches(ch, "InfoScreen"))
-    {
-      char buf[512];
-      snprintf(buf, 512, _("Information about %s"), package.Name());
-
-      vscreen_widget *w=make_info_screen(package, visible_version());
-      insert_main_widget(w, buf, "");
-    }
+    show_information();
   else if(bindings->key_matches(ch, "Changelog") &&
 	  !visible_version().end())
-    view_changelog(visible_version());
+    show_changelog();
   else if(bindings->key_matches(ch, "InstallSingle"))
     {
       if((*apt_cache_file)[package].CandidateVerIter(*apt_cache_file).end())
@@ -331,6 +350,33 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
       else
 	delete grp;
     }
+  else if(bindings->key_matches(ch, "BugReport"))
+    {
+      // Try to report a bug on the package.  (ew quoting ew)
+      string cmd=string("reportbug '")+package.Name()+"'";
+
+      // Default to reporting a bug on the current version.
+      pkgCache::VerIterator ver=package.CurrentVer();
+      if(ver.end())
+	ver=visible_version();
+      if(ver.end())
+	ver=package.VersionList();
+
+      if(!ver.end())
+	cmd+=string(" -V '")+ver.VerStr()+"'";
+
+      vscreen_suspend();
+
+      apt_cache_file->ReleaseLock();
+
+      printf(_("Reporting a bug in %s:\n"), package.Name());
+
+      system(cmd.c_str());
+
+      vscreen_resume();
+
+      apt_reload_cache(gen_progress_bar(), true);
+    }
   else if(bindings->key_matches(ch, "DpkgReconfigure"))
     // Don't bother with my internal su-to-root stuff here, since I don't
     // need to touch the package lists in the subprocess.
@@ -367,9 +413,9 @@ bool pkg_item::dispatch_char(chtype ch, vs_tree *owner)
 	      getchar();
 
 	      vscreen_resume();
-
-	      apt_reload_cache(gen_progress_bar(), true);
 	    }
+
+	  apt_reload_cache(gen_progress_bar(), true);
 	}
     }
   else if(bindings->key_matches(ch, "EditHier"))
@@ -395,13 +441,7 @@ void pkg_item::dispatch_mouse(short id, int x, mmask_t bstate, vs_tree *owner)
 	       BUTTON3_DOUBLE_CLICKED | BUTTON4_DOUBLE_CLICKED |
 	       BUTTON1_TRIPLE_CLICKED | BUTTON2_TRIPLE_CLICKED |
 	       BUTTON3_TRIPLE_CLICKED | BUTTON4_TRIPLE_CLICKED))
-    {
-      char buf[512];
-      snprintf(buf, 512, _("Information about %s"), package.Name());
-
-      vscreen_widget *w=make_info_screen(package, visible_version());
-      insert_main_widget(w, buf, "");
-    }
+    show_information();
 }
 
 bool pkg_item::matches(string s) const
@@ -413,9 +453,11 @@ pkgCache::VerIterator pkg_item::visible_version(const pkgCache::PkgIterator &pkg
 {
   pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
 
-  if(pkg.CurrentVer().end())
-    return state.CandidateVerIter(*apt_cache_file);
-  else if(state.Install())
+  pkgCache::VerIterator candver=state.CandidateVerIter(*apt_cache_file);
+
+  if(!candver.end())
+    return candver;
+  else if(pkg.CurrentVer().end() && state.Install())
     return state.InstVerIter(*apt_cache_file);
   // Return the to-be-installed version
   else
