@@ -72,7 +72,7 @@ public:
     win->getmaxyx(height, width);
     pkg_columnizer::setup_columns();
 
-    std::string disp=pkg_with_subtree_columnizer(get_expanded(),
+    std::string disp=pkg_with_subtree_columnizer(vs_subtree<child, sorter>::get_expanded(),
 						 hierarchical,
 						 get_package(),
 						 visible_version(),
