@@ -70,6 +70,16 @@ void pkg_subtree::hold(undo_group *undo)
   (*apt_cache_file)->end_action_group(undo);
 }
 
+void pkg_subtree::keep(undo_group *undo)
+{
+  (*apt_cache_file)->begin_action_group();
+
+  for(child_iterator i=get_children_begin(); i!=get_children_end(); i++)
+    (*i)->keep(undo);
+
+  (*apt_cache_file)->end_action_group(undo);
+}
+
 void pkg_subtree::remove(undo_group *undo)
 {
   (*apt_cache_file)->begin_action_group();
