@@ -1,6 +1,6 @@
 // pkg_ver_item.cc
 //
-//  Copyright 1999,2000,2001 Daniel Burrows
+//  Copyright 1999,2000,2001, 2005 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
 #include <string>
 
 #include <apt-pkg/configuration.h>
+#include <apt-pkg/init.h> // For the APT version
 #include <apt-pkg/policy.h>
 #include <apt-pkg/strutl.h>
 
@@ -402,6 +403,17 @@ column_disposition pkg_ver_columnizer::setup_column(int type)
 	snprintf(buf, 100, "%i", count);
 	return column_disposition(buf, 0);
       }
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+    case trust_state:
+      {
+	if(ver.end())
+	  return column_disposition(" ", 0);
+	else if(package_trusted(ver))
+	  return column_disposition(" ", 0);
+	else
+	  return column_disposition("U", 0);
+      }
+#endif
     default:
       return pkg_columnizer::setup_column(type);
     }
