@@ -46,6 +46,7 @@ public:
 	       bool _show_pkg_name=false)
     :version(_version), show_pkg_name(_show_pkg_name), sig(_sig) {}
 
+  pkgCache::PkgIterator get_package() const {return version.ParentPkg();}
   const pkgCache::VerIterator &get_version() const {return version;}
 
   virtual int get_normal_attr();
@@ -56,14 +57,19 @@ public:
 
   virtual void select(undo_group *undo);
   virtual void hold(undo_group *undo);
+  virtual void keep(undo_group *undo);
   virtual void remove(undo_group *undo);
   virtual void purge(undo_group *undo);
   virtual void reinstall(undo_group *undo);
   virtual void set_auto(bool isauto, undo_group *undo);
 
+  virtual void forbid_version(undo_group *undo);
+
   virtual void highlighted(vs_tree *win);
   virtual void unhighlighted(vs_tree *win);
 
+  void show_information();
+
   bool dispatch_char(chtype ch, vs_tree *owner);
 
   pkg_ver_item *get_sig();
