@@ -11,6 +11,7 @@
 #include "menu_redirect.h"
 #include "pkg_columnizer.h"
 #include "reason_fragment.h"
+#include "trust.h"
 #include "ui.h"
 
 #include <vscreen/fragment.h>
@@ -26,6 +27,7 @@
 #include <generic/config_signal.h>
 
 #include <apt-pkg/error.h>
+#include <apt-pkg/init.h> // For the APT version.
 
 #include <sigc++/bind.h>
 #include <sigc++/object_slot.h>
@@ -131,7 +133,23 @@ public:
     else
       newdesc="";
 
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+    fragment *frag=make_desc_fragment(newdesc);
+
+    fragment *untrusted_frag;
+
+    if(pkg.end() || ver.end())
+      untrusted_frag=NULL;
+    else
+      untrusted_frag=make_untrusted_warning(ver);
+
+    if(untrusted_frag == NULL)
+      set_fragment(frag);
+    else
+      set_fragment(fragf("%F%n%F", untrusted_frag, frag));
+#else
     set_fragment(make_desc_fragment(newdesc));
+#endif
   }
 };
 
