@@ -18,9 +18,10 @@
 #include <list>
 #include <string>
 
+class menu_redirect;
+class package_view_item;
 class pkg_grouppolicy_factory;
 class vscreen_widget;
-class package_view_item;
 
 // The available types of items are:
 //
@@ -53,6 +54,8 @@ struct package_view_item
   std::string columns_cfg;
   // For static items, the name of a configuration setting which will be
   // tracked; the columns will be regenerated when this is modified.
+  std::string columns_cfg_default;
+  // The default value for columns_cfg.
   int row, col, w, h;
   int xopts, yopts;
   // obvious table stuff
@@ -110,6 +113,7 @@ struct package_view_item
  *
  *  \param format a description of the layout of the view
  *  \param mainwidget the widget which will be used to fill in MAINWIDGET items
+ *  \param menu_handler the handler to use for menu commands.
  *  \param sig the signal which will be called when a package is selected by
  *             the main widget.  If this is \b NULL, nothing will be connected
  *             to it.
@@ -122,6 +126,7 @@ struct package_view_item
  */
 vscreen_widget *make_package_view(std::list<package_view_item> &format,
 				  vscreen_widget *mainwidget,
+				  menu_redirect *menu_handler,
 				  pkg_signal *sig, desc_signal *desc_sig,
 				  bool show_reason_first);
 
