@@ -70,13 +70,15 @@ struct ver_ptr_cmp
 
 /** Generate a fragment describing the packages providing a given package.
  *
- *  "installed" acts as a filter: if \b true, only providees which are
- *  to be installed (whether they are currently installed or not) are
- *  shown; otherwise, only providees which are currently installed and
- *  to be removed are shown.  This corresponds to appropriate behavior
- *  for Conflicts and other dependencies.
+ *  \param pkg the package whose providers will be described.
+ *  \param ignpkg a package which should not be shown as a provider.
+ *
+ *  \param installed a filter: if \b true, only providees which are
+ *  installed, or which will be installed, are shown; otherwise, all
+ *  providees are shown.
  */
 fragment *prvfrag(pkgCache::PkgIterator pkg,
+		  pkgCache::PkgIterator ignpkg,
 		  bool installed)
 {
   // All packages providing the given package name are listed.
@@ -97,8 +99,11 @@ fragment *prvfrag(pkgCache::PkgIterator pkg,
   for(pkgCache::PrvIterator P=pkg.ProvidesList();
       !P.end(); ++P)
     {
-      providing_versions.insert(P.OwnerVer());
-      providing_packages.insert(P.OwnerPkg());
+      if(P.OwnerPkg()!=ignpkg)
+	{
+	  providing_versions.insert(P.OwnerVer());
+	  providing_packages.insert(P.OwnerPkg());
+	}
     }
 
   for(set<pkgCache::PkgIterator>::const_iterator P=providing_packages.begin();
@@ -110,14 +115,16 @@ fragment *prvfrag(pkgCache::PkgIterator pkg,
 
       if(installed)
 	{
-	  if(instver.end())
+	  if(instver.end() && currver.end())
 	    continue;
 	}
+#if 0
       else
 	{
 	  if(!(!currver.end() && instver.end()))
 	    continue;
 	}
+#endif
 
       if(!currver.end() &&
 	 providing_versions.find(currver)!=providing_versions.end())
@@ -166,7 +173,7 @@ fragment *dep_singlefrag(pkgCache::PkgIterator pkg,
 {
   fragment *verfrag;
 
-  pkgCache::VerIterator instver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);	
+  pkgCache::VerIterator instver=(*apt_cache_file)[pkg].InstVerIter(*apt_cache_file);
 
   if(!dep.TargetVer())
     verfrag=text_fragment("", A_NORMAL);
@@ -218,14 +225,26 @@ fragment *dep_singlefrag(pkgCache::PkgIterator pkg,
   else
     sec=string(sec, 0, sec.find('/'));
 
-  return fragf("%F%s%F%F",
+  bool available=false;
+
+  for(pkgCache::VerIterator i=dep.TargetPkg().VersionList(); !i.end(); i++)
+    if(_system->VS->CheckDep(i.VerStr(), dep->CompareOp, dep.TargetVer()))
+      available=true;
+
+  for(pkgCache::PrvIterator i=dep.TargetPkg().ProvidesList(); !i.end(); i++)
+    if(_system->VS->CheckDep(i.ProvideVersion(), dep->CompareOp, dep.TargetVer()))
+      available=true;
+
+  return fragf("%F%s%F%F%s",
 	       text_fragment(dep.TargetPkg().Name(),
 			     pkg_item::pkg_attr(dep.TargetPkg(),
 						A_NORMAL)),
 	       sec.empty() || sec=="main"?"":(" ["+sec+']').c_str(),
 	       verfrag,
 	       prvfrag(dep.TargetPkg(),
-		       dep->Type==pkgCache::Dep::Conflicts));
+		       dep.ParentPkg(),
+		       dep->Type==pkgCache::Dep::Conflicts),
+	       available?"":(string(" [")+_("UNAVAILABLE")+"]").c_str());
 }
 
 /** Generate a fragment describing the OR group that contains the
@@ -430,15 +449,6 @@ fragment *reason_fragment(const pkgCache::PkgIterator &pkg, bool &breakage)
     }
 
 
-  int dsize=(instver.end()?0:instver->InstalledSize)
-    -(pkg.CurrentVer().end()?0:pkg.CurrentVer()->InstalledSize);
-  if(dsize>0)
-    fragments.push_back(wrapbox(fragf(_("  This will use %sB of disk space."),
-				      SizeToStr(dsize).c_str())));
-  else if(dsize<0)
-    fragments.push_back(wrapbox(fragf(_("  This will free %sB of disk space."),
-				      SizeToStr(-dsize).c_str())));
-
   if(!reasons.empty())
     fragments.push_back(sequence_fragment(newline_fragment(),
 					  newline_fragment(),
