@@ -1,6 +1,6 @@
 // ui.cc
 //
-//  Copyright 2000-2002 Daniel Burrows <Daniel_Burrows@brown.edu>
+//  Copyright 2000-2005 Daniel Burrows <dburrows@debian.org>
 //
 //  Various UI glue-code and routines.
 
@@ -13,6 +13,7 @@
 #include <apt-pkg/clean.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
+#include <apt-pkg/init.h> // For the APT version number.
 
 #include <errno.h>
 #include <fcntl.h>
@@ -736,6 +737,184 @@ void install_or_remove_packages()
   active_download=false;
 }
 
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+/** Hack to get around the broken tables in this aptitude version.
+ *
+ *  This widget always requests the maximum size available.
+ */
+class hacked_vs_layout:public vs_text_layout
+{
+public:
+  hacked_vs_layout(fragment *f):vs_text_layout(f) {}
+
+  size size_request()
+  {
+    unsigned int screen_width=rootwin.getmaxx();
+
+    return size(screen_width, height_for_width(screen_width));
+  }
+};
+
+/** This variant of vs_dialog_yesno was backported from an
+ *  experimental aptitude, so it's compiled out for sarge's aptitude
+ *  (unless we switch to apt 0.6), but it's not expected to cause any
+ *  problems.
+ */
+vscreen_widget *vs_dialog_yesno(vscreen_widget *widget,
+				Slot0Arg yesslot,
+				string yeslabel,
+				Slot0Arg noslot,
+				string nolabel,
+				int attr,
+				bool deflt)
+{
+  vs_center *center=new vs_center;
+
+  vs_table *table=new vs_table;
+  table->set_bg(attr);
+
+  vs_button *yesbutton=new vs_button(yeslabel);
+  yesbutton->set_bg(attr);
+  vs_button *nobutton=new vs_button(nolabel);
+  nobutton->set_bg(attr);
+
+  yesbutton->pressed.connect(slot(*center, &vscreen_widget::destroy));
+  nobutton->pressed.connect(slot(*center, &vscreen_widget::destroy));
+
+  if(yesslot)
+    yesbutton->pressed.connect(yesslot);
+  if(noslot)
+    nobutton->pressed.connect(noslot);
+
+  table->connect_key("Yes", &global_bindings, yesbutton->pressed.slot());
+  table->connect_key("No", &global_bindings, nobutton->pressed.slot());
+  table->connect_key("Cancel", &global_bindings, nobutton->pressed.slot());
+
+  table->add_widget(widget, 0, 0, 1, 2, true, true);
+  table->add_widget_opts(yesbutton, 1, 0, 1, 1, vs_table::SHRINK|vs_table::ALIGN_CENTER, 0);
+  table->add_widget_opts(nobutton, 1, 1, 1, 1, vs_table::SHRINK|vs_table::ALIGN_CENTER, 0);
+
+  widget->show();
+  yesbutton->show();
+  nobutton->show();
+
+  if(deflt)
+    table->focus_widget(yesbutton);
+  else
+    table->focus_widget(nobutton);
+
+  vs_frame *frame=new vs_frame(table);
+  frame->set_bg(attr);
+
+  center->add_widget(frame);
+
+  return center;
+}
+
+vscreen_widget *vs_dialog_yesno(fragment *msg,
+				Slot0Arg yesslot,
+				std::string yeslabel,
+				Slot0Arg noslot,
+				std::string nolabel,
+				int attr,
+				bool scrollbar,
+				bool deflt)
+{
+  vscreen_widget *w;
+
+  if(scrollbar)
+    {
+      vs_table *t=new vs_table;
+      w=t;
+
+      vs_text_layout *l=new hacked_vs_layout(msg);
+      vs_scrollbar *s=new vs_scrollbar(vs_scrollbar::VERTICAL);
+
+      l->set_bg(attr);
+      s->set_bg(attr);
+
+      t->add_widget(l, 0, 0, 1, 1, true, true);
+      t->add_widget_opts(s, 0, 1, 1, 1,
+			 vs_table::ALIGN_RIGHT,
+			 vs_table::ALIGN_CENTER | vs_table::EXPAND);
+
+      l->location_changed.connect(slot(*s, &vs_scrollbar::set_slider));
+      s->scrollbar_interaction.connect(slot(*l, &vs_text_layout::scroll));
+    }
+  else
+    w=new hacked_vs_layout(msg);
+
+  w->set_bg(attr);
+
+  return vs_dialog_yesno(w, yesslot, yeslabel, noslot, nolabel, attr, deflt);
+}
+
+vscreen_widget *vs_dialog_yesno(fragment *msg,
+				Slot0Arg yesslot,
+				Slot0Arg noslot,
+				int attr,
+				bool scrollbar,
+				bool deflt)
+{
+  return vs_dialog_yesno(msg, yesslot, _("Yes"), noslot, _("No"), attr,
+			 scrollbar, deflt);
+}
+
+/** Make sure that no trust violations are about to be committed.  If
+ *  any are, warn the user and give them a chance to fix the
+ *  situation.
+ *
+ *  Should this warning be displayed when a package is selected instead?
+ */
+static void check_package_trust()
+{
+  vector<pkgCache::VerIterator> untrusted;
+
+  for(pkgCache::PkgIterator pkg=(*apt_cache_file)->PkgBegin();
+      !pkg.end(); ++pkg)
+    {
+      pkgDepCache::StateCache &state=(*apt_cache_file)[pkg];
+
+      if(state.Install())
+	{
+	  pkgCache::VerIterator curr=pkg.CurrentVer();
+	  pkgCache::VerIterator cand=state.InstVerIter(*apt_cache_file);
+
+	  if((curr.end() || package_trusted(curr)) &&
+	     !package_trusted(cand))
+	    untrusted.push_back(cand);
+	}
+    }
+
+  if(!untrusted.empty())
+    {
+      vector<fragment *> frags;
+
+      frags.push_back(wrapbox(fragf(_("%BWARNING%b: untrusted versions of the following packages will be installed!%n%n"
+				      "Untrusted packages could %Bcompromise your system's security%b.  "
+				      "You should only proceed with the installation if you are certain that this is what you want to do.%n%n"))));
+
+      for(vector<pkgCache::VerIterator>::const_iterator i=untrusted.begin();
+	  i!=untrusted.end(); ++i)
+	frags.push_back(clipbox(fragf("  %F %s [version %s]%n",
+				      text_fragment("*", get_color("Bullet")),
+				      i->ParentPkg().Name(), i->VerStr())));
+
+      main_stacked->add_visible_widget(vs_dialog_yesno(sequence_fragment(frags),
+						       SigC::slot(install_or_remove_packages),
+						       _("Really Continue"),
+						       NULL,
+						       _("Abort Installation"),
+						       get_color("DefaultWidgetBackground"),
+						       true,
+						       false),
+				       true);
+    }
+  else
+    install_or_remove_packages();
+}
+#endif
+
 static void actually_do_package_run();
 static void fix_pkgs_and_do_package_run();
 
@@ -849,14 +1028,22 @@ static void actually_do_package_run()
 	    }
 
 	  if(getuid()==0  || !aptcfg->FindB(PACKAGE "::Warn-Not-Root", true))
+#if APT_PKG_MAJOR >=3 && APT_PKG_MINOR >= 5
+	    check_package_trust();
+#else
 	    install_or_remove_packages();
+#endif
 	  else
 	{
 	  popup_widget(vs_dialog_yesno(_("Installing/removing packages requires administrative privileges, which\nyou currently do not have.  Would you like to change to the root account?\n"),
 				       bind(SigC::slot(&do_su_to_root),
 					    "-i"),
 				       _("Become root"),
+#if APT_PKG_MAJOR >= 3 && APT_PKG_MINOR >= 5
+				       SigC::slot(&check_package_trust),
+#else
 				       SigC::slot(&install_or_remove_packages),
+#endif
 				       _("Don't become root"),
 				       get_color("Error")));
 	}
