@@ -15,6 +15,7 @@
 
 #include <string>
 
+#include <vscreen/bool_marshal.h>
 #include <vscreen/slotarg.h>
 #include <vscreen/vs_editline.h>
 // Bleah..this needs to be included so histories work
@@ -41,6 +42,79 @@ extern SigC::Signal0<void> file_quit;
 // call vscreen_update().
 extern SigC::Signal0<void> package_states_changed;
 
+/** Used to determine whether a target is available for the "package actions".
+ */
+extern SigC::Signal0<bool, MarshalOr> package_menu_enabled;
+
+/** Emitted for Package -> Install. */
+extern SigC::Signal0<bool, MarshalOr> package_install;
+
+/** Emitted for Package -> Remove. */
+extern SigC::Signal0<bool, MarshalOr> package_remove;
+
+/** Emitted for Package -> Purge. */
+extern SigC::Signal0<bool, MarshalOr> package_purge;
+
+/** Emitted for Package -> Hold. */
+extern SigC::Signal0<bool, MarshalOr> package_hold;
+
+/** Emitted for Package -> Keep. */
+extern SigC::Signal0<bool, MarshalOr> package_keep;
+
+/** Emitted for Package -> Mark Auto */
+extern SigC::Signal0<bool, MarshalOr> package_mark_auto;
+
+/** Emitted for Package -> Mark Manual */
+extern SigC::Signal0<bool, MarshalOr> package_unmark_auto;
+
+/** Tests whether Package -> Forbid Version is enabled. */
+extern SigC::Signal0<bool, MarshalOr> package_forbid_enabled;
+
+/** Emitted for Package -> Forbid. */
+extern SigC::Signal0<bool, MarshalOr> package_forbid;
+
+/** Tests whether Package -> Package Information is enabled. */
+extern SigC::Signal0<bool, MarshalOr> package_information_enabled;
+
+/** Emitted for Package -> Package Information. */
+extern SigC::Signal0<bool, MarshalOr> package_information;
+
+/** Tests whether Package -> Changelog is enabled. */
+extern SigC::Signal0<bool, MarshalOr> package_changelog_enabled;
+
+/** Emitted for Package -> Changelog. */
+extern SigC::Signal0<bool, MarshalOr> package_changelog;
+
+/** Tests whether Search -> Find is enabled. */
+extern SigC::Signal0<bool, MarshalOr> find_search_enabled;
+
+/** Tests whether Search -> Find Again is enabled. */
+extern SigC::Signal0<bool, MarshalOr> find_research_enabled;
+
+/** Tests whether Search -> Limit is enabled. */
+extern SigC::Signal0<bool, MarshalOr> find_limit_enabled;
+
+/** Tests whether Search -> Cancel Limit is enabled. */
+extern SigC::Signal0<bool, MarshalOr> find_cancel_limit_enabled;
+
+/** Tests whether Searc -> Find Broken is enabled. */
+extern SigC::Signal0<bool, MarshalOr> find_broken_enabled;
+
+/** Emitted for Search -> Find. */
+extern SigC::Signal0<bool, MarshalOr> find_search;
+
+/** Emitted for Search -> Find Again. */
+extern SigC::Signal0<bool, MarshalOr> find_research;
+
+/** Emitted for Search -> Limit. */
+extern SigC::Signal0<bool, MarshalOr> find_limit;
+
+/** Emitted for Search -> Cancel Limit. */
+extern SigC::Signal0<bool, MarshalOr> find_cancel_limit;
+
+/** Emitted for Search -> Find Broken. */
+extern SigC::Signal0<bool, MarshalOr> find_broken;
+
 /*****************************************************************************/
 
 void ui_init();
@@ -60,6 +134,9 @@ void add_main_widget(vscreen_widget *w, const std::string &menuref,
 void insert_main_widget(vscreen_widget *w, const std::string &menuref,
 			const std::string &menudesc);
 
+/** Returns the currently active main widget. */
+vscreen_widget *active_main_widget();
+
 // Generates a progress bar.
 vs_progress *gen_progress_bar();
 
@@ -74,12 +151,15 @@ fragment *wrapbox(fragment *contents);
  *            the screen).
  *  \param title             if a new view is generated, this string is
  *                           used as its title.
+ *  \param longtitle         if a new view is generated, this string is
+ *                           used as its long title.
  *  \param abortslot         the slot to trigger if the download is aborted.
  *
  *  \return the new download manager.
  */
 download_manager *gen_download_progress(bool force_noninvasive,
 					const std::string &title,
+					const std::string &longtitle,
 					Slot0Arg abortslot);
 
 // Asks the user for simple input (the question will appear in a "minibuffer"
