@@ -22,18 +22,21 @@
 #ifndef VS_PROGRESS_H
 #define VS_PROGRESS_H
 
-#include "vs_tree.h"
+#include "vscreen/vs_tree.h"
+#include "vscreen/vscreen_widget.h"
 
 #include <apt-pkg/progress.h>
 
-class vs_progress:public vs_tree_status, public OpProgress
+class vs_progress:public vscreen_widget, public OpProgress
 {
 public:
-  vs_progress(vs_tree *_owner):vs_tree_status(_owner) {}
+  virtual void paint();
+  virtual void Update();
+  virtual void Done();
 
-  void display();
-  void Update();
-  void Done();
+  size size_request() {return size(40, 1);}
+  bool get_cursorvisible();
+  point get_cursorloc();
 };
 
 #endif
