@@ -58,6 +58,9 @@ void init_key_tables()
   keynames["space"]=' ';
   rev_keynames[' ']="space";
 
+  keynames["comma"]=',';
+  rev_keynames[',']="comma";
+
   keynames["break"]=KEY_BREAK;
   rev_keynames[KEY_BREAK]="break";
   keynames["down"]=KEY_DOWN;
