@@ -49,6 +49,10 @@ class keybindings
   HASH_NAMESPACE::hash_map<std::string, keybinding> keymap;
 
   keybindings *parent;
+
+  // It's way too easy to accidentally invoke the automatic copy
+  // constructor instead of the real one.
+  keybindings(const keybindings &_parent);
 public:
   keybindings(keybindings *_parent=NULL):parent(_parent) {}
 
