@@ -6,11 +6,13 @@
 
 #include "config/keybindings.h"
 
+#include <sigc++/object_slot.h>
+
 vs_button::vs_button(std::string _label)
   :label(_label)
 {
-  focussed.connect(slot(this, &vs_button::accept_focus));
-  unfocussed.connect(slot(this, &vs_button::lose_focus));
+  focussed.connect(slot(*this, &vs_button::accept_focus));
+  unfocussed.connect(slot(*this, &vs_button::lose_focus));
 }
 
 bool vs_button::focus_me()
