@@ -2,12 +2,14 @@
 
 #include "vs_center.h"
 
+#include <sigc++/object_slot.h>
+
 vs_center::vs_center(vscreen_widget *w)
 {
   set_subwidget(w);
   set_bg(-1);
 
-  do_layout.connect(slot(this, &vs_center::layout_me));
+  do_layout.connect(slot(*this, &vs_center::layout_me));
 }
 
 size vs_center::size_request()
@@ -35,9 +37,3 @@ void vs_center::layout_me()
 	get_subwidget()->alloc_size(0, 0, 0, 0);
     }
 }
-
-void vs_center::paint()
-{
-  if(get_subwidget() && get_subwidget()->get_visible())
-    get_subwidget()->display();
-}
