@@ -4,12 +4,14 @@
 
 #include "config/colors.h"
 
+#include <sigc++/object_slot.h>
+
 vs_frame::vs_frame(vscreen_widget *w)
   :vs_bin()
 {
   set_subwidget(w);
 
-  do_layout.connect(slot(this, &vs_frame::layout_me));
+  do_layout.connect(slot(*this, &vs_frame::layout_me));
 }
 
 size vs_frame::size_request()
