@@ -6,6 +6,7 @@
 
 #include <sigc++/bind.h>
 #include <sigc++/object_slot.h>
+#include <sigc++/retype_return.h>
 
 #include <algorithm>
 #include <ctype.h>
@@ -17,20 +18,45 @@ keybindings *vs_menu::bindings=NULL;
 vs_menu_info::vs_menu_info(item_types type, const char *name, const char *binding,
 			   const char *description, SigC::Slot0<void> slot)
   :item_type(type), item_name(name), item_binding(binding),
-   item_description(description), item_slot(slot)
+   item_description(description), item_slot(slot), item_enabled(NULL)
 {
 }
 
 vs_menu_info::vs_menu_info(item_types type, const char *name, const char *binding,
 			   const char *description, SigC::Slot0<void> *slot)
   :item_type(type), item_name(name), item_binding(binding),
-   item_description(description), item_slot(slot)
+   item_description(description), item_slot(slot), item_enabled(NULL)
+{
+}
+
+vs_menu_info::vs_menu_info(item_types type, const char *name, const char *binding,
+			   const char *description, SigC::Slot0<void> slot,
+			   SigC::Slot0<bool> enabled)
+  :item_type(type), item_name(name), item_binding(binding),
+   item_description(description), item_slot(slot), item_enabled(enabled)
+{
+}
+
+vs_menu_info::vs_menu_info(item_types type, const char *name, const char *binding,
+			   const char *description, SigC::Slot0<void> *slot,
+			   SigC::Slot0<bool> enabled)
+  :item_type(type), item_name(name), item_binding(binding),
+   item_description(description), item_slot(slot), item_enabled(enabled)
+{
+}
+
+vs_menu_info::vs_menu_info(item_types type, const char *name, const char *binding,
+			   const char *description, SigC::Slot0<bool> slot,
+			   SigC::Slot0<bool> enabled)
+  :item_type(type), item_name(name), item_binding(binding),
+   item_description(description), item_slot(SigC::retype_return<void>(slot)),
+   item_enabled(enabled)
 {
 }
 
 vs_menu_info::vs_menu_info(item_types type)
   :item_type(type), item_name(NULL), item_binding(NULL),
-   item_description(NULL), item_slot(NULL)
+   item_description(NULL), item_slot(NULL), item_enabled(NULL)
 {
 }
 
@@ -47,6 +73,14 @@ vs_menu_item::vs_menu_item(const string &_title, const string &_binding,
       }
 }
 
+bool vs_menu_item::is_enabled()
+{
+  if(enabled.empty())
+    return !selected.empty();
+  else
+    return enabled();
+}
+
 vs_menu::vs_menu()
   :vscreen_widget(), cursorloc(0), req_width(2)
 {
@@ -76,6 +110,10 @@ vs_menu::vs_menu(int x, int y, int w, vs_menu_info *inf)
 
 	    if(inf->item_slot)
 	      newitem->selected.connect(inf->item_slot);
+
+	    if(inf->item_enabled)
+	      newitem->enabled.connect(inf->item_enabled);
+
 	    append_item(newitem);
 	  }
 	  break;
@@ -99,7 +137,22 @@ vs_menu::vs_menu(int x, int y, int w, vs_menu_info *inf)
 void vs_menu::append_item(vs_menu_item *newitem)
 {
   items.push_back(newitem);
-  req_width=newitem?max<int>(newitem->get_title().size()+2,req_width):req_width;
+  if(newitem)
+    {
+      string::size_type shortcutsize=0;
+      string::size_type menusize=0;
+
+      for(string::size_type i=0; i<newitem->get_title().size(); ++i)
+	if(newitem->get_title()[i]!='^')
+	  ++menusize;
+
+      if(newitem->get_binding().empty())
+	shortcutsize=0;
+      else
+	shortcutsize=global_bindings.keyname(newitem->get_binding()).size()+1;
+
+      req_width=max<int>(menusize+2+shortcutsize,req_width);
+    }
 
   if(get_visible())
     {
@@ -123,24 +176,12 @@ void vs_menu::remove_item(vs_menu_item *item)
   items.pop_back();
 
   if(items.size()==0)
-    cursorloc=0;
+    set_cursor(0);
   else if(idx==cursorloc)
-    {
-      if(cursorloc>=items.size())
-	cursorloc=items.size()-1;
-
-      while(cursorloc<items.size() && !items[cursorloc])
-	++cursorloc;
-
-      while(cursorloc>0 && !items[cursorloc])
-	--cursorloc;
-    }
+    set_cursor(prev_selectable(next_selectable(items.size()-1)));
 
   if(get_visible())
-    {
-      vscreen_queuelayout();
-      vscreen_update();
-    }
+    vscreen_queuelayout();
 }
 
 size vs_menu::size_request()
@@ -148,15 +189,36 @@ size vs_menu::size_request()
   return size(req_width, items.size()+2);
 }
 
+void vs_menu::highlight_current()
+{
+  if(cursorloc>=0 && cursorloc<items.size())
+    item_highlighted(items[cursorloc]);
+  else
+    item_highlighted(NULL);
+}
+
+void vs_menu::set_cursor(itemlist::size_type pos)
+{
+  if(cursorloc!=pos)
+    {
+      cursorloc=pos;
+      highlight_current();
+
+      if(get_visible())
+	vscreen_update();
+    }
+}
+
 void vs_menu::appear()
 {
-  cursorloc=0;
-  item_highlighted(items[cursorloc]);
+  cursorloc=next_selectable(0);
+
+  highlight_current();
 }
 
 void vs_menu::disappear()
 {
-  item_highlighted(NULL);
+  set_cursor(items.size());
 }
 
 bool vs_menu::focus_me()
@@ -164,68 +226,99 @@ bool vs_menu::focus_me()
   return true;
 }
 
+bool vs_menu::selectable(itemlist::size_type pos)
+{
+  return pos>=0 && pos<items.size() && items[pos] && items[pos]->is_enabled();
+}
+
+vs_menu::itemlist::size_type vs_menu::next_selectable(itemlist::size_type pos)
+{
+  if(pos<0 || pos>=items.size())
+    pos=0;
+
+  while(pos<items.size() && (!items[pos] ||
+			     !items[pos]->is_enabled()))
+    ++pos;
+
+  return pos;
+}
+
+vs_menu::itemlist::size_type vs_menu::prev_selectable(itemlist::size_type pos)
+{
+  if(pos<0 || pos>=items.size())
+    pos=items.size()-1;
+
+  while(pos>=0 && pos<items.size() && (!items[pos] ||
+				       !items[pos]->is_enabled()))
+    --pos;
+
+  if(pos<0 || pos>=items.size())
+    pos=items.size();
+
+  return pos;
+}
+
+void vs_menu::sanitize_cursor(bool forward)
+{
+  if(forward)
+    // Double-invoking next_selectable will search from the
+    // start of the menu if searching forwards fails.
+    cursorloc=next_selectable(next_selectable(cursorloc));
+  else
+    cursorloc=prev_selectable(prev_selectable(cursorloc));
+
+  highlight_current();
+}
 
 bool vs_menu::handle_char(chtype ch)
 {
-  assert(cursorloc>=0 && cursorloc<items.size());
+  // This will ensure that the cursor is in bounds if possible, and that
+  // if it is in bounds, a "real" item is selected.
+  sanitize_cursor(true);
 
   if(bindings->key_matches(ch, "Up"))
     {
       if(cursorloc>0)
-      {
-	itemlist::size_type newloc=cursorloc-1;
-	while(newloc>=0 && !items[newloc])
-	  newloc--;
-	if(newloc>=0)
-	  cursorloc=newloc;
+	{
+	  itemlist::size_type newloc=prev_selectable(cursorloc-1);
 
-	item_highlighted(items[cursorloc]);
-      }
+	  if(newloc>=0 && newloc<items.size())
+	    set_cursor(newloc);
 
-      vscreen_update();
+	  vscreen_update();
+	}
     }
   else if(bindings->key_matches(ch, "Down"))
     {
       if(cursorloc<items.size()-1)
 	{
-	  itemlist::size_type newloc=cursorloc+1;
-	  while(newloc<items.size() && !items[newloc])
-	    newloc++;
-	  if(newloc<items.size())
-	    cursorloc=newloc;
+	  itemlist::size_type newloc=next_selectable(cursorloc+1);
 
-	  item_highlighted(items[cursorloc]);
+	  if(newloc>=0 && newloc<items.size())
+	    set_cursor(newloc);
+
+	  vscreen_update();
 	}
-
-      vscreen_update();
     }
-  // FIXME?  -- if the last item is a separator Weird Things will happen
   else if(bindings->key_matches(ch, "Begin"))
-    {
-      cursorloc=0;
-      item_highlighted(items[cursorloc]);
-
-      vscreen_update();
-    }
-  // Similarly, should this be a FIXME?
+    set_cursor(next_selectable(0));
   else if(bindings->key_matches(ch, "End"))
-    {
-      cursorloc=items.size()-1;
-      item_highlighted(items[cursorloc]);
-
-      vscreen_update();
-    }
+    set_cursor(prev_selectable(items.size()-1));
   else if(bindings->key_matches(ch, "Confirm"))
     {
+      itemlist::size_type selected=cursorloc;
+
       menus_goaway();
       item_highlighted(NULL);
 
-      items[cursorloc]->selected();
+      if(selectable(selected))
+	items[selected]->selected();
     }
   else
     {
       for(itemlist::iterator i=items.begin(); i!=items.end(); i++)
-	if(*i && toupper(ch)==toupper((*i)->get_hotkey()))
+	if(*i && (*i)->is_enabled() &&
+	   toupper(ch)==toupper((*i)->get_hotkey()))
 	  {
 	    menus_goaway();
 	    item_highlighted(NULL);
@@ -249,7 +342,7 @@ void vs_menu::dispatch_mouse(short id, int x, int y, int z, mmask_t bmask)
     {
       itemlist::size_type num=y-1;
 
-      if(num>=0 && num<items.size() && items[num])
+      if(selectable(num))
 	{
 	  menus_goaway();
 	  item_highlighted(NULL);
@@ -261,13 +354,8 @@ void vs_menu::dispatch_mouse(short id, int x, int y, int z, mmask_t bmask)
     {
       itemlist::size_type num=y-1;
 
-      if(num>=0 && num<items.size() && items[num])
-	{
-	  cursorloc=num;
-
-	  item_highlighted(items[cursorloc]);
-	  vscreen_update();
-	}
+      if(selectable(num))
+	set_cursor(num);
     }
 }
 
@@ -287,6 +375,9 @@ void vs_menu::paint()
 
   string::size_type maxlen=width-2;
 
+  // Make sure that whatever is selected is really selectable.
+  sanitize_cursor(true);
+
   for(itemlist::size_type i=0; i<items.size(); i++)
     if(items[i])
       {
@@ -299,10 +390,14 @@ void vs_menu::paint()
 	string title=items[i]->get_title();
 	string righttext=items[i]->get_binding().empty()?"":global_bindings.keyname(items[i]->get_binding());
 
+	bool enabled=items[i]->is_enabled();
+
 	if(i==cursorloc)
 	  attrset(get_color("HighlightedMenuEntry"));
-	else
+	else if(enabled)
 	  attrset(get_color("MenuEntry"));
+	else
+	  attrset(get_color("DisabledMenuEntry"));
 
 	move(1+i, 1);
 
@@ -312,7 +407,7 @@ void vs_menu::paint()
 	  {
 	    while(titleloc<title.size() && title[titleloc]=='^')
 	      {
-		boldthis=true;
+		boldthis=enabled;
 		++titleloc;
 	      }
 
@@ -329,24 +424,22 @@ void vs_menu::paint()
 		  addch((unsigned char) righttext[rightloc++]);
 	      }
 	    else if(boldthis)
-		{
-		  attron(A_BOLD);
-		  addch((unsigned char) title[titleloc++]);
-		  attroff(A_BOLD);
-		  boldthis=false;
-		}
+	      {
+		addch(((unsigned char) title[titleloc++])|A_BOLD);
+		boldthis=false;
+	      }
 	    else
 	      addch((unsigned char) title[titleloc++]);
 	  }
       }
-  else
-    {
-      attrset(get_color("MenuBorder"));
-      mvaddch(1+i, 0, ACS_LTEE);
-      for(int j=1; j<width-1; j++)
-	addch(ACS_HLINE);
-      addch(ACS_RTEE);
-    }
+    else
+      {
+	attrset(get_color("MenuBorder"));
+	mvaddch(1+i, 0, ACS_LTEE);
+	for(int j=1; j<width-1; j++)
+	  addch(ACS_HLINE);
+	addch(ACS_RTEE);
+      }
 
   attrset(get_color("MenuBorder"));
   for(int i=items.size()+1; i<height-1; i++)
@@ -366,11 +459,15 @@ void vs_menu::paint()
 
 bool vs_menu::get_cursorvisible()
 {
-  return true;
+  sanitize_cursor(true);
+
+  return cursorloc>=0 && cursorloc<items.size();
 }
 
 point vs_menu::get_cursorloc()
 {
+  sanitize_cursor(true);
+
   return point(0, 1+cursorloc);
 }
 
