@@ -4,12 +4,18 @@
 
 #include <vscreen.h>
 
+int vs_scrollbar::get_slider()
+{
+  int width = dir==HORIZONTAL?getmaxx():getmaxy();
+  return max==0?-1:(width-1)*val/max;
+}
+
 void vs_scrollbar::paint()
 {
   if(dir==HORIZONTAL)
     {
       int width=getmaxx();
-      int thumbloc=max==0?-1:width*val/max;
+      int thumbloc=get_slider();
 
       for(int x=0; x<width; x++)
 	if(x==thumbloc)
@@ -20,7 +26,7 @@ void vs_scrollbar::paint()
   else
     {
       int height=getmaxy();
-      int thumbloc=max==0?-1:height*val/max;
+      int thumbloc=get_slider();
 
       for(int y=0; y<height; y++)
 	if(y==thumbloc)
@@ -55,3 +61,19 @@ point vs_scrollbar::get_cursorloc()
 {
   return point(0, 0);
 }
+
+void vs_scrollbar::dispatch_mouse(short id,
+				  int x, int y, int z,
+				  mmask_t bstate)
+{
+  int slider_loc=get_slider();
+  int mloc = dir==HORIZONTAL?x:y;
+
+  if(slider_loc!=-1)
+    {
+      if(mloc>slider_loc)
+	scrollbar_interaction(false);
+      else
+	scrollbar_interaction(true);
+    }
+}
