@@ -20,6 +20,12 @@ private:
 
   int max, val;
   // The current slider maximum and value (FIXME: use floats?)
+
+  /** Get the current X or Y location of the slider in the widget. 
+   *
+   *  \return the slider location, or -1 if it is not visible.
+   */
+  int get_slider();
 public:
   vs_scrollbar(direction _dir, int _val, int _max)
     :dir(_dir), max(_max), val(_val) {}
@@ -31,8 +37,15 @@ public:
   size size_request();
   bool get_cursorvisible();
   point get_cursorloc();
+  void dispatch_mouse(short id, int x, int y, int z, mmask_t bstate);
 
   void set_slider(int newval, int newmax);
+
+  /** This signal is emitted if the user "pages up" or "pages down"
+   *  via the scrollbar.  Its argument is \b true for a "page up" and
+   *  \b false for a "page down".
+   */
+  SigC::Signal1<void, bool> scrollbar_interaction;
 };
 
 #endif
