@@ -114,7 +114,7 @@ void vs_table::add_widget_opts(vscreen_widget *w, int row_start, int col_start,
       focus=children.end();
       focus--;
 
-      if(get_isfocussed())
+      if(focus!=children.end() && get_isfocussed())
 	focus->w->focussed();
     }
 
@@ -123,7 +123,7 @@ void vs_table::add_widget_opts(vscreen_widget *w, int row_start, int col_start,
 
 void vs_table::hide_widget(vscreen_widget *w)
 {
-  if(w==focus->w)
+  if(focus!=children.end() && w==focus->w)
     {
       if(get_isfocussed())
 	focus->w->unfocussed();
@@ -1127,5 +1127,5 @@ void vs_table::dispatch_mouse(short id, int x, int y, int z, mmask_t bstate)
 
 void vs_table::init_bindings()
 {
-  bindings=new keybindings(global_bindings);
+  bindings=new keybindings(&global_bindings);
 }
