@@ -14,7 +14,7 @@ void vs_treeitem::highlighted(vs_tree *win)
 }
 
 void vs_treeitem::paint(vs_tree *win, int y, bool hierarchical, string str,
-			int depth_shift=2)
+			int depth_shift)
 {
   int width, height;
   int basex=hierarchical?depth_shift*get_depth():0;
