@@ -35,6 +35,7 @@
 #include "vs_pager.h"
 #include "vs_statuschoice.h"
 #include "vs_table.h"
+#include "vs_text_layout.h"
 #include "vs_tree.h"
 
 #include "config/keybindings.h"
@@ -53,7 +54,7 @@
 #include <sys/time.h>
 
 #ifdef HAVE_LIBPTHREAD
-static pthread_mutex_t vscreen_mutex=PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+static pthread_mutex_t vscreen_mutex;
 #endif
 
 using namespace std;
@@ -247,7 +248,6 @@ void vscreen_init()
   rootwin.keypad(true);
 
   global_bindings.set("Quit", quitkey);
-  global_bindings.set("AbortScreen", "x"[0]);
   global_bindings.set("Cycle", '\t');
   global_bindings.set("Refresh", KEY_CTRL("l"[0]));
 
@@ -278,8 +278,8 @@ void vscreen_init()
   global_bindings.set("Help", helpkey);
   global_bindings.set("ToggleMenuActive", menutogglekey);
   global_bindings.set("PushButton", ' ');
-  global_bindings.set("Yes", "y"[0]);
-  global_bindings.set("No", "n"[0]);
+  global_bindings.set("Yes", _("yes_key")[0]);
+  global_bindings.set("No", _("no_key")[0]);
 
   global_bindings.set("ToggleExpanded", KEY_ENTER);
   global_bindings.set("ExpandAll", '[');
@@ -291,20 +291,21 @@ void vscreen_init()
   vs_menubar::init_bindings();
   vs_pager::init_bindings();
   vs_statuschoice::init_bindings();
-  vs_tree::init_bindings();
   vs_table::init_bindings();
+  vs_text_layout::init_bindings();
+  vs_tree::init_bindings();
 
   set_color("Error", COLOR_WHITE, COLOR_RED, A_BOLD);
 
   set_color("DefaultWidgetBackground", COLOR_WHITE, COLOR_BLACK, 0);
 
   set_color("ScreenHeaderColor", COLOR_WHITE, COLOR_BLUE, A_BOLD);
-  set_color("ScreenBackgroundColor", COLOR_WHITE, COLOR_BLACK, 0);
   set_color("ScreenStatusColor", COLOR_WHITE, COLOR_BLUE, A_BOLD);
 
   set_color("MenuEntry", COLOR_WHITE, COLOR_BLUE, 0);
   set_color("MenuBorder", COLOR_WHITE, COLOR_BLUE, A_BOLD);
-  set_color("HighlightedMenuEntry", COLOR_BLUE, COLOR_WHITE, A_BOLD);
+  set_color("HighlightedMenuEntry", COLOR_WHITE, COLOR_BLUE, A_BOLD|A_REVERSE);
+  set_color("DisabledMenuEntry", COLOR_BLACK, COLOR_BLUE, A_DIM);
   set_color("MenuBar", COLOR_WHITE, COLOR_BLUE, A_BOLD);
   set_color("HighlightedMenuBar", COLOR_BLUE, COLOR_WHITE, A_BOLD);
 
@@ -317,6 +318,14 @@ void vscreen_init()
   if(toplevel)
     vscreen_settoplevel(toplevel);
 
+  pthread_mutexattr_t vscreen_mutex_attr;
+  pthread_mutexattr_init(&vscreen_mutex_attr);
+  pthread_mutexattr_settype(&vscreen_mutex_attr, PTHREAD_MUTEX_RECURSIVE);
+
+  pthread_mutex_init(&vscreen_mutex, &vscreen_mutex_attr);
+
+  pthread_mutexattr_destroy(&vscreen_mutex_attr);
+
   vscreen_install_sighandlers();
 }
 
