@@ -49,7 +49,7 @@ struct rect
 
 class keybindings;
 
-class vscreen_widget:public SigC::Object
+class vscreen_widget:virtual public SigC::Object
 {
   friend class vs_container;
 
